# -*- coding: utf-8 -*-

import os
import oss2
from .common import *
from oss2 import SizedFileAdapter, determine_part_size
from oss2.models import OSS_TRAFFIC_LIMIT, PartInfo

OBJECT_SIZE_1MB = (1 * 1024 * 1024)
LIMIT_100KB = (100 * 1024 * 8)

class TestTrafficLimit(OssTestCase):
    def test_put_object(self):
        key = 'traffic-limit-test-put-object'
        content = b'a' * OBJECT_SIZE_1MB

        # Put object with payer setting, should be successful.
        headers = dict()
        headers[OSS_TRAFFIC_LIMIT] = str(LIMIT_100KB);

        start_time_sec = int(time.time())
        result = self.bucket.put_object(key, content, headers=headers)
        self.assertEqual(result.status, 200)
        end_time_sec = int(time.time())

        # Calculate expensed time
        expense_time_sec = end_time_sec - start_time_sec

        # Theoretical time is 1MB/100KB = 10s, set the minimum theoretical time to 10*0.7s
        theoretical_exepnse_min = 10 * 0.7

        # Compare to minimum theoretical time
        self.assertEqual((expense_time_sec>theoretical_exepnse_min), True)
        self.bucket.delete_object(key)

    def test_get_object(self):
        key = 'traffic-limit-test-get-object'
        content = b'a' * OBJECT_SIZE_1MB
        file_name = key + '.txt'

        result = self.bucket.put_object(key, content)

        headers = dict()
        headers[OSS_TRAFFIC_LIMIT] = str(LIMIT_100KB);

        # Get object with traffic limit
        start_time_sec = int(time.time())
        result = self.bucket.get_object_to_file(key, file_name, headers=headers)
        self.assertEqual(result.status, 200)
        end_time_sec = int(time.time())

        os.remove(file_name)
        self.bucket.delete_object(key)

        # Calculate expensed time
        expense_time_sec = end_time_sec - start_time_sec

        # Theoretical time is 1MB/100KB = 10s, set the minimum theoretical time to 10*0.7s
        theoretical_exepnse_min = 10 * 0.7

        # Compare to minimum theoretical time
        self.assertEqual((expense_time_sec>theoretical_exepnse_min), True)

    def test_append_object(self):
        key = 'traffic-limit-test-apend-object'
        content = b'a' * OBJECT_SIZE_1MB

        headers = dict()
        headers[OSS_TRAFFIC_LIMIT] = str(LIMIT_100KB);

        # Append object with traffic limit
        start_time_sec = int(time.time())
        result = self.bucket.append_object(key, 0, content, headers=headers)
        self.assertEqual(result.status, 200)
        end_time_sec = int(time.time())

        self.bucket.delete_object(key)

        # Calculate expensed time
        expense_time_sec = end_time_sec - start_time_sec

        # Theoretical time is 1MB/100KB = 10s, set the minimum theoretical time to 10*0.7s
        theoretical_exepnse_min = 10 * 0.7

        # Compare to minimum theoretical time
        self.assertEqual((expense_time_sec>theoretical_exepnse_min), True)

    def test_resumable_upload(self):
        key = 'traffic-limit-test-resumble-upload-object'
        pathname = self._prepare_temp_file_with_size(OBJECT_SIZE_1MB)

        headers = dict()
        headers[OSS_TRAFFIC_LIMIT] = str(LIMIT_100KB);

        # Resumable upload object smaller than multipart_threshold with traffic limit
        start_time_sec = int(time.time())
        result = oss2.resumable_upload(self.bucket, key, pathname, multipart_threshold=(OBJECT_SIZE_1MB*2), headers=headers, num_threads=1)
        self.assertEqual(result.status, 200)
        end_time_sec = int(time.time())

        self.bucket.delete_object(key)

        # Calculate expensed time
        expense_time_sec = end_time_sec - start_time_sec
        # Theoretical time is 1MB/100KB = 10s, set the minimum theoretical time to 10*0.7s
        theoretical_exepnse_min = 10 * 0.7
        # Compare to minimum theoretical time
        self.assertEqual((expense_time_sec>theoretical_exepnse_min), True)

        # Resumable upload object bigger than multipart_threshold with traffic limit
        start_time_sec = int(time.time())
        result = oss2.resumable_upload(self.bucket, key, pathname, multipart_threshold=(OBJECT_SIZE_1MB-1024), headers=headers, num_threads=1)
        self.assertEqual(result.status, 200)
        end_time_sec = int(time.time())

        self.bucket.delete_object(key)

        # Calculate expensed time
        expense_time_sec = end_time_sec - start_time_sec
        # Theoretical time is 1MB/100KB = 10s, set the minimum theoretical time to 10*0.7s
        theoretical_exepnse_min = 10 * 0.7
        # Compare to minimum theoretical time
        self.assertEqual((expense_time_sec>theoretical_exepnse_min), True)

    def test_upload_part(self):
        key = 'traffic-limit-test-resumble-upload-object'
        # Create tmp file 2MB
        file_name = self._prepare_temp_file_with_size(OBJECT_SIZE_1MB * 2)

        total_size = os.path.getsize(file_name)
        # Determine part size is 1MB
        part_size = determine_part_size(total_size, preferred_size=(1024*1024))     

        # Init part
        upload_id = self.bucket.init_multipart_upload(key).upload_id
        parts = []  

        headers = dict()
        headers[OSS_TRAFFIC_LIMIT] = str(LIMIT_100KB);    

        #  Upload part
        with open(file_name, 'rb') as fileobj:
            part_number = 1
            offset = 0
            while offset < total_size:
                num_to_upload = min(part_size, total_size - offset)

                # Upload part with traffic limit setting
                start_time_sec = int(time.time())
                result = self.bucket.upload_part(key, upload_id, part_number,
                                            SizedFileAdapter(fileobj, num_to_upload),
                                            headers=headers)
                end_time_sec = int(time.time())

                # Calculate expensed time
                expense_time_sec = end_time_sec - start_time_sec
                # Theoretical time is 1MB/100KB = 10s, set the minimum theoretical time to 10*0.7s
                theoretical_exepnse_min = 10 * 0.7
                # Compare to minimum theoretical time
                self.assertEqual((expense_time_sec>theoretical_exepnse_min), True)
                self.assertEqual((expense_time_sec > theoretical_exepnse_min), True)

                parts.append(PartInfo(part_number, result.etag))
                offset += num_to_upload
                part_number += 1        

        result = self.bucket.complete_multipart_upload(key, upload_id, parts)
        self.assertEqual(result.status, 200)

    def test_resumable_download(self):
        key = 'traffic-limit-test-resumble-download-object'
        content = b'a' * OBJECT_SIZE_1MB
        file_name = key + '.txt'

        # Put object
        self.bucket.put_object(key, content)

        headers = dict()
        headers[OSS_TRAFFIC_LIMIT] = str(LIMIT_100KB)

        # Resumable download object smaller than multiget_threshold with traffic limit setting.
        start_time_sec = int(time.time())
        oss2.resumable_download(self.bucket, key, file_name, multiget_threshold=(OBJECT_SIZE_1MB*2), num_threads=1, headers=headers)
        self.assertFileContent(file_name, content)
        end_time_sec = int(time.time())

        os.remove(file_name)

        # Calculate expensed time
        expense_time_sec = end_time_sec - start_time_sec
        # Theoretical time is 1MB/100KB = 10s, set the minimum theoretical time to 10*0.7s
        theoretical_exepnse_min = 10 * 0.7
        # Compare to minimum theoretical time
        self.assertEqual((expense_time_sec>theoretical_exepnse_min), True)

        # Resumable download object bigger than multiget_threshold with traffic limit setting.
        start_time_sec = int(time.time())
        oss2.resumable_download(self.bucket, key, file_name, multiget_threshold=(OBJECT_SIZE_1MB-1024), num_threads=1, headers=headers)
        self.assertFileContent(file_name, content)
        end_time_sec = int(time.time())

        os.remove(file_name)
        self.bucket.delete_object(key)

        # Calculate expensed time
        expense_time_sec = end_time_sec - start_time_sec
        # Theoretical time is 1MB/100KB = 10s, set the minimum theoretical time to 10*0.7s
        theoretical_exepnse_min = 10 * 0.7
        # Compare to minimum theoretical time
        self.assertEqual((expense_time_sec>theoretical_exepnse_min), True)

    def test_put_object_with_signed_url(self):
        key = 'traffic-limit-test-put-object-signed-url'
        file_name = self._prepare_temp_file_with_size(OBJECT_SIZE_1MB)

        # Create url with taffic limit setting.
        params = dict()
        params[OSS_TRAFFIC_LIMIT] = str(LIMIT_100KB);  
        url = self.bucket.sign_url('PUT', key, 60, params=params)

        # Put object with url form file
        start_time_sec = int(time.time())
        result = self.bucket.put_object_with_url_from_file(url, file_name)
        self.assertEqual(result.status, 200)
        end_time_sec = int(time.time())

        # Check file size
        result = self.bucket.head_object(key)
        self.assertEqual(result.content_length, OBJECT_SIZE_1MB)

        self.bucket.delete_object(key)

        # Calculate expensed time
        expense_time_sec = end_time_sec - start_time_sec

        # Theoretical time is 1MB/100KB = 10s, set the minimum theoretical time to 10*0.7s
        theoretical_exepnse_min = 10 * 0.7

        # Compare to minimum theoretical time
        self.assertEqual((expense_time_sec>theoretical_exepnse_min), True)

    def test_get_object_with_signed_url(self):
        key = 'traffic-limit-test-get-object-signed-url'
        content = b'a' * OBJECT_SIZE_1MB
        file_name = key + '.txt'

        # Put file 
        result = self.bucket.put_object(key, content);
        self.assertEqual(result.status, 200)

        # Create url with taffic limit setting.
        params = dict()
        params[OSS_TRAFFIC_LIMIT] = str(LIMIT_100KB);  
        url = self.bucket.sign_url('GET', key, 60, params=params)

        # Get object to file with url 
        start_time_sec = int(time.time())
        result = self.bucket.get_object_with_url_to_file(url, file_name)
        self.assertEqual(result.status, 200)
        end_time_sec = int(time.time())

        # Check file size
        file_size = os.stat(file_name).st_size
        self.assertEqual(file_size, OBJECT_SIZE_1MB)

        os.remove(file_name)
        self.bucket.delete_object(key)

        # Calculate expensed time
        expense_time_sec = end_time_sec - start_time_sec

        # Theoretical time is 1MB/100KB = 10s, set the minimum theoretical time to 10*0.7s
        theoretical_exepnse_min = 10 * 0.7

        # Compare to minimum theoretical time
        self.assertEqual((expense_time_sec>theoretical_exepnse_min), True)


if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-

import os
import time

import oss2


# 以下代码展示了视频直播相关接口的用法。


# 首先初始化AccessKeyId、AccessKeySecret、Endpoint等信息。
# 通过环境变量获取，或者把诸如“<您的AccessKeyId>”替换成真实的AccessKeyId等。
#
# 以杭州区域为例，Endpoint是：
#   http://oss-cn-shenzhen.aliyuncs.com 或
#   https://oss-cn-shenzhen.aliyuncs.com
access_key_id = os.getenv('OSS_TEST_ACCESS_KEY_ID', '<您的AccessKeyId>')
access_key_secret = os.getenv('OSS_TEST_ACCESS_KEY_SECRET', '<您的AccessKeySecret>')
bucket_name = os.getenv('OSS_TEST_BUCKET', '<您的Bucket>')
endpoint = os.getenv('OSS_TEST_ENDPOINT', '<您的访问域名>')


# 确认上面的参数都填写正确了
for param in (access_key_id, access_key_secret, bucket_name, endpoint):
    assert '<' not in param, '请设置参数：' + param


# 创建Bucket对象，所有直播相关的接口都可以通过Bucket对象来进行
bucket = oss2.Bucket(oss2.Auth(access_key_id, access_key_secret), endpoint, bucket_name)


# 创建一个直播频道。
# 频道的名称是test_rtmp_live。直播生成的m3u8文件叫做test.m3u8，该索引文件包含3片ts文件，每片ts文件的时长为5秒（这只是一个建议值，具体的时长取决于关键帧）。
channel_name = 'test_rtmp_live'
playlist_name = 'test.m3u8'
create_result = bucket.create_live_channel(
        channel_name,
        oss2.models.LiveChannelInfo(
            status = 'enabled',
            description = '测试使用的直播频道',
            target = oss2.models.LiveChannelInfoTarget(
                playlist_name = playlist_name,
                frag_count = 3,
                frag_duration = 5)))

# 创建直播频道之后拿到推流用的play_url（rtmp推流的url，如果Bucket不是公共读写权限那么还需要带上签名，见下文示例）和观流用的publish_url（推流产生的m3u8文件的url）。
publish_url = create_result.publish_url
play_url = create_result.play_url

# 创建好直播频道之后调用get_live_channel可以得到频道相关的信息。
get_result = bucket.get_live_channel(channel_name)
print(get_result.description)
print(get_result.status)
print(get_result.target.type)
print(get_result.target.frag_count)
print(get_result.target.frag_duration)
print(get_result.target.playlist_name)

# 拿到推流地址和观流地址之后就可以向OSS推流和观流。如果Bucket的权限不是公共读写，那么还需要对推流做签名，如果Bucket是公共读写的，那么可以直接用publish_url推流。
# 这里的expires是一个相对时间，指的是从现在开始这次推流过期的秒数。
# params是一个dict类型的参数，表示用户自定义的参数。所有的参数都会参与签名。
# 拿到这个签过名的signed_url就可以使用推流工具直接进行推流，一旦连接上OSS之后超过上面的expires流也不会断掉，OSS仅在每次推流连接的时候检查expires是否合法。
expires = 3600
signed_url = bucket.sign_rtmp_url(channel_name, playlist_name, expires)

# 创建好直播频道，如果想把这个频道禁用掉（断掉正在推的流或者不再允许向一个地址推流），应该使用put_live_channel_status接口，将频道的status改成“disabled”，如果要将一个禁用状态的频道启用，那么也是调用这个接口，将status改成“enabled”。
bucket.put_live_channel_status(channel_name, 'enabled')
bucket.put_live_channel_status(channel_name, 'disabled')

# 对创建好的频道，可以使用LiveChannelIterator来进行列举已达到管理的目的。
# prefix可以按照前缀过滤list出来的频道。
# max_keys表示迭代器内部一次list出来的频道的最大数量，这个值最大不能超过1000，不填写的话默认为100。

prefix = ''
max_keys = 1000

for info in oss2.LiveChannelIterator(bucket, prefix, max_keys=max_keys):
    print(info.name)

# 对于正在推流的频道调用get_live_channel_stat可以获得流的状态信息。
# 如果频道正在推流，那么stat_result中的所有字段都有意义。
# 如果频道闲置或者处于“disabled”状态，那么status为“Idle”或“Disabled”，其他字段无意义。
stat_result = bucket.get_live_channel_stat(channel_name)
print(stat_result.status)
print(stat_result.remote_addr)
print(stat_result.connected_time)
print(stat_result.video)
print(stat_result.audio)

# 如果想查看一个频道历史推流记录，可以调用get_live_channel_history。目前最多可以看到10次推流的记录
history_result = bucket.get_live_channel_history(channel_name)
print(len(history_result.records))

# 如果希望利用直播推流产生的ts文件生成一个点播列表，可以使用post_vod_playlist方法。
# 指定起始时间为当前时间减去60秒，结束时间为当前时间，这意味着将生成一个长度为60秒的点播视频。
# 播放列表指定为“vod_playlist.m3u8”，也就是说这个接口调用成功之后会在OSS上生成一个名叫“vod_playlist.m3u8”的播放列表文件。

end_time = int(time.time()) - 60
start_time = end_time - 3600
bucket.post_vod_playlist(channel_name,
                         playlist_name,
                         start_time = start_time,
                         end_time = end_time)

# 如果想查看指定时间段内的播放列表，可以使用get_vod_playlist
result = bucket.get_vod_playlist(channel_name, start_time=start_time, end_time=end_time)
print("playlist:", result.playlist)

# 如果一个直播频道已经不打算再使用了，那么可以调用delete_live_channel来删除频道。
bucket.delete_live_channel(channel_name)



# -*- coding: utf-8 -*-

import os
import oss2
from .common import *
from oss2.models import ReplicationRule


class TestBucketReplication(OssTestCase):
    def setUp(self):
        OssTestCase.setUp(self)
        source_bucket_name = OSS_BUCKET + '-test-replica-source-' + random_string(10)
        self.source_bucket = oss2.Bucket(oss2.make_auth(OSS_ID, OSS_SECRET, OSS_AUTH_VERSION), OSS_ENDPOINT, source_bucket_name)
        self.source_bucket.create_bucket()

        self.replica_endpoint = "oss-ap-south-1"
        self.transfer_type = 'oss_acc'
        self.replica_bucket_name = OSS_BUCKET + "-test-replica-dest-" + random_string(10)
        self.replica_bucket = oss2.Bucket(oss2.make_auth(OSS_ID, OSS_SECRET, OSS_AUTH_VERSION), self.replica_endpoint + '.aliyuncs.com', self.replica_bucket_name)
        self.replica_bucket.create_bucket()

        import time
        time.sleep(3)

        index = OSS_INVENTORY_BUCKET_DESTINATION_ARN.rfind('/')
        self.sync_role_name = OSS_INVENTORY_BUCKET_DESTINATION_ARN[index+1:] if index != -1 else OSS_INVENTORY_BUCKET_DESTINATION_ARN

    def test_replication_with_full_parameter(self):
        rule_id = 'test-id'
        prefix_list = ['prefix-1', 'prefix-2']
        replica_config = ReplicationRule(rule_id=rule_id,
                                         prefix_list=prefix_list,
                                         action_list=[ReplicationRule.ALL],
                                         target_bucket_name=self.replica_bucket_name,
                                         target_bucket_location=self.replica_endpoint,
                                         target_transfer_type=self.transfer_type,
                                         is_enable_historical_object_replication=False,
                                         sync_role_name=self.sync_role_name,
                                         replica_kms_keyid=OSS_CMK,
                                         sse_kms_encrypted_objects_status=ReplicationRule.ENABLED)
        result = self.source_bucket.put_bucket_replication(replica_config)
        self.assertEqual(200, result.status)

        result = self.source_bucket.get_bucket_replication()
        self.assertEqual(1, len(result.rule_list))
        rule = result.rule_list[0]
        self.assertEqual(rule_id, rule.rule_id)
        self.assertEqual(self.replica_bucket_name, rule.target_bucket_name)
        self.assertEqual(self.replica_endpoint, rule.target_bucket_location)
        self.assertEqual(self.transfer_type, rule.target_transfer_type)
        self.assertEqual(2, len(rule.prefix_list))
        self.assertTrue('prefix-1' in rule.prefix_list)
        self.assertTrue('prefix-2' in rule.prefix_list)
        self.assertEqual(1, len(rule.action_list))
        self.assertEqual(ReplicationRule.ALL, rule.action_list[0])
        self.assertFalse(rule.is_enable_historical_object_replication)
        self.assertEqual(ReplicationRule.STARTING, rule.status)
        self.assertEqual(self.sync_role_name, rule.sync_role_name)
        self.assertEqual(OSS_CMK, rule.replica_kms_keyid)
        self.assertEqual(ReplicationRule.ENABLED, rule.sse_kms_encrypted_objects_status)

        result = self.source_bucket.get_bucket_replication_progress(rule_id)
        progress = result.progress
        self.assertEqual(self.replica_bucket_name, progress.target_bucket_name)
        self.assertEqual(self.replica_endpoint, progress.target_bucket_location)
        self.assertEqual(rule_id, progress.rule_id)
        self.assertEqual(self.transfer_type, progress.target_transfer_type)
        self.assertEqual(2, len(rule.prefix_list))
        self.assertTrue('prefix-1' in rule.prefix_list)
        self.assertTrue('prefix-2' in rule.prefix_list)
        self.assertEqual(1, len(progress.action_list))
        self.assertEqual(ReplicationRule.ALL, progress.action_list[0])
        self.assertFalse(progress.is_enable_historical_object_replication)
        self.assertEqual(ReplicationRule.STARTING, progress.status)

        self.assertEqual(0, progress.historical_object_progress)
        self.assertIsNone(progress.new_object_progress)

        result = self.source_bucket.delete_bucket_replication(rule_id)
        self.assertEqual(200, result.status)

        result = self.source_bucket.get_bucket_replication()
        self.assertEqual(ReplicationRule.CLOSING, result.rule_list[0].status)

        result = self.source_bucket.get_bucket_replication_progress(rule_id)
        self.assertEqual(ReplicationRule.CLOSING, result.progress.status)

    def test_get_replication_location(self):
        result = self.source_bucket.get_bucket_replication_location()
        self.assertTrue(len(result.location_list) > 0)
        self.assertTrue(len(result.location_transfer_type_list) > 0)
        for location in result.location_list:
            self.assertIsNotNone(location)
        for location_transfer_type in result.location_transfer_type_list:
            self.assertIsNotNone(location_transfer_type.location)
            self.assertIsNotNone(location_transfer_type.transfer_type)

    def test_put_without_rule_id(self):
        prefix_list = ['prefix-1', 'prefix-2']
        replica_config = ReplicationRule(prefix_list=prefix_list,
                                         target_bucket_name=self.replica_bucket_name,
                                         target_bucket_location=self.replica_endpoint,
                                         target_transfer_type=self.transfer_type,
                                         sync_role_name=self.sync_role_name,
                                         replica_kms_keyid=OSS_CMK,
                                         sse_kms_encrypted_objects_status='Enabled')
        result = self.source_bucket.put_bucket_replication(replica_config)
        self.assertEqual(200, result.status)

        result = self.source_bucket.get_bucket_replication()
        self.assertEqual(1, len(result.rule_list))
        rule = result.rule_list[0]
        self.assertIsNotNone(rule.rule_id)
        self.assertEqual(self.replica_bucket_name, rule.target_bucket_name)
        self.assertEqual(self.replica_endpoint, rule.target_bucket_location)
        self.assertEqual(self.transfer_type, rule.target_transfer_type)
        self.assertEqual(2, len(rule.prefix_list))
        self.assertTrue('prefix-1' in rule.prefix_list)
        self.assertTrue('prefix-2' in rule.prefix_list)
        self.assertEqual(1, len(rule.action_list))
        self.assertEqual(ReplicationRule.ALL, rule.action_list[0])
        self.assertTrue(rule.is_enable_historical_object_replication)
        self.assertEqual(ReplicationRule.STARTING, rule.status)
        self.assertEqual(self.sync_role_name, rule.sync_role_name)
        self.assertEqual(OSS_CMK, rule.replica_kms_keyid)
        self.assertEqual('Enabled', rule.sse_kms_encrypted_objects_status)
        rule_id = rule.rule_id

        result = self.source_bucket.delete_bucket_replication(rule_id)
        self.assertEqual(200, result.status)

    def test_put_replica_without_prefix(self):
        rule_id = 'test-id'
        replica_config = ReplicationRule(rule_id=rule_id,
                                         target_bucket_name=self.replica_bucket_name,
                                         target_bucket_location=self.replica_endpoint,
                                         target_transfer_type=self.transfer_type,
                                         sync_role_name=self.sync_role_name,
                                         replica_kms_keyid=OSS_CMK,
                                         sse_kms_encrypted_objects_status='Enabled')
        result = self.source_bucket.put_bucket_replication(replica_config)
        self.assertEqual(200, result.status)

        result = self.source_bucket.get_bucket_replication()
        self.assertEqual(1, len(result.rule_list))
        rule = result.rule_list[0]
        self.assertIsInstance(rule.prefix_list, list)
        self.assertEqual(0, len(rule.prefix_list))

        result = self.source_bucket.get_bucket_replication_progress(rule_id)
        progress = result.progress
        self.assertEqual(0, len(progress.prefix_list))

        result = self.source_bucket.delete_bucket_replication(rule_id)
        self.assertEqual(200, result.status)

    def test_put_replica_without_sync_role(self):
        rule_id = 'test-id'
        prefix_list = ['prefix-1', 'prefix-2']
        replica_config = ReplicationRule(rule_id=rule_id,
                                         prefix_list=prefix_list,
                                         target_bucket_name=self.replica_bucket_name,
                                         target_bucket_location=self.replica_endpoint,
                                         target_transfer_type=self.transfer_type,
                                         replica_kms_keyid=OSS_CMK,
                                         sse_kms_encrypted_objects_status='Enabled')
        result = self.source_bucket.put_bucket_replication(replica_config)
        self.assertEqual(200, result.status)

        result = self.source_bucket.get_bucket_replication()
        self.assertEqual(1, len(result.rule_list))
        rule = result.rule_list[0]
        self.assertEqual(rule_id, rule.rule_id)
        self.assertIsNone(rule.sync_role_name)
        self.assertIsNone(rule.replica_kms_keyid)
        self.assertIsNone(rule.sse_kms_encrypted_objects_status)

        result = self.source_bucket.delete_bucket_replication(rule_id)
        self.assertEqual(200, result.status)

    def test_put_replica_without_action(self):
        rule_id = 'test-id'
        replica_config = ReplicationRule(rule_id=rule_id,
                                         target_bucket_name=self.replica_bucket_name,
                                         target_bucket_location=self.replica_endpoint,
                                         is_enable_historical_object_replication=True)
        self.source_bucket.put_bucket_replication(replica_config)

        result = self.source_bucket.get_bucket_replication()
        self.assertEqual(1, len(result.rule_list))
        rule = result.rule_list[0]
        self.assertTrue(rule.is_enable_historical_object_replication)
        self.assertEqual(ReplicationRule.ALL, rule.action_list[0])

    def test_put_replica_with_multi_action(self):
        rule_id = 'test-id'
        action_list = [ReplicationRule.PUT, ReplicationRule.DELETE, ReplicationRule.ABORT]
        replica_config = ReplicationRule(rule_id=rule_id,
                                         target_bucket_name=self.replica_bucket_name,
                                         target_bucket_location=self.replica_endpoint,
                                         action_list=action_list)
        self.source_bucket.put_bucket_replication(replica_config)

        result = self.source_bucket.get_bucket_replication()
        self.assertEqual(1, len(result.rule_list))
        rule = result.rule_list[0]
        self.assertEqual(3, len(rule.action_list))
        self.assertTrue('PUT' in rule.action_list)
        self.assertTrue('DELETE' in rule.action_list)
        self.assertTrue('ABORT' in rule.action_list)

    def test_put_multi_replica_rule(self):

        rule_id = 'test-id'
        action_list = [ReplicationRule.PUT, ReplicationRule.DELETE, ReplicationRule.ABORT]
        replica_config = ReplicationRule(rule_id=rule_id,
                                         target_bucket_name=self.replica_bucket_name,
                                         target_bucket_location=self.replica_endpoint,
                                         action_list=action_list)
        self.source_bucket.put_bucket_replication(replica_config)

        rule_id_2 = 'test-id-2'
        action_list = [ReplicationRule.PUT, ReplicationRule.DELETE, ReplicationRule.ABORT]
        replica_config = ReplicationRule(rule_id=rule_id_2,
                                         target_bucket_name=self.replica_bucket_name,
                                         target_bucket_location=self.replica_endpoint,
                                         action_list=action_list)
        self.assertRaises(oss2.exceptions.BucketReplicationAlreadyExist, self.source_bucket.put_bucket_replication, replica_config)

    def test_kms_encrypted_status_disabled(self):
        rule_id = 'test-id'
        prefix_list = ['prefix-1', 'prefix-2']
        replica_config = ReplicationRule(rule_id=rule_id,
                                         prefix_list=prefix_list,
                                         target_bucket_name=self.replica_bucket_name,
                                         target_bucket_location=self.replica_endpoint,
                                         target_transfer_type=self.transfer_type,
                                         sync_role_name=self.sync_role_name,
                                         replica_kms_keyid=OSS_CMK,
                                         sse_kms_encrypted_objects_status=ReplicationRule.DISABLED)
        result = self.source_bucket.put_bucket_replication(replica_config)
        self.assertEqual(200, result.status)

        result = self.source_bucket.get_bucket_replication()
        self.assertEqual(1, len(result.rule_list))
        rule = result.rule_list[0]
        self.assertEqual(rule_id, rule.rule_id)
        self.assertEqual(self.replica_bucket_name, rule.target_bucket_name)
        self.assertEqual(self.replica_endpoint, rule.target_bucket_location)
        self.assertEqual(self.transfer_type, rule.target_transfer_type)
        self.assertEqual(2, len(rule.prefix_list))
        self.assertTrue('prefix-1' in rule.prefix_list)
        self.assertTrue('prefix-2' in rule.prefix_list)
        self.assertEqual(1, len(rule.action_list))
        self.assertEqual(ReplicationRule.ALL, rule.action_list[0])
        self.assertTrue(rule.is_enable_historical_object_replication)
        self.assertEqual(ReplicationRule.STARTING, rule.status)
        self.assertEqual(self.sync_role_name, rule.sync_role_name)
        self.assertEqual(ReplicationRule.DISABLED, rule.sse_kms_encrypted_objects_status)
        # kms will be None
        self.assertIsNone(rule.replica_kms_keyid)


    def test_wrong_kms_encrypted_status(self):

        right_rule = ReplicationRule(target_bucket_name=self.replica_bucket_name,
                                         target_bucket_location=self.replica_endpoint,
                                         target_transfer_type=self.transfer_type,
                                         sse_kms_encrypted_objects_status=ReplicationRule.ENABLED)

        self.assertRaises(oss2.exceptions.ClientError, ReplicationRule, target_bucket_name=self.replica_bucket_name,
                          target_bucket_location=self.replica_endpoint, target_transfer_type=self.transfer_type,
                          sse_kms_encrypted_objects_status='wrong')

    def test_wrong_enable_historical_setting(self):
        right_rule = ReplicationRule(target_bucket_name=self.replica_bucket_name,
                                     target_bucket_location=self.replica_endpoint,
                                     target_transfer_type=self.transfer_type,
                                     is_enable_historical_object_replication=True)

        self.assertRaises(oss2.exceptions.ClientError, ReplicationRule, target_bucket_name=self.replica_bucket_name,
                          target_bucket_location=self.replica_endpoint, target_transfer_type=self.transfer_type,
                          is_enable_historical_object_replication='wrong')

    def test_replica_rule_id(self):
        self.assertRaises(oss2.exceptions.NoSuchReplicationRule, self.source_bucket.get_bucket_replication_progress, 'wrong')


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

import json
import os

from aliyunsdkcore import client
from aliyunsdksts.request.v20150401 import AssumeRoleRequest

import oss2


# 以下代码展示了STS的用法，包括角色扮演获取临时用户的密钥、使用临时用户的密钥访问OSS。

# STS入门教程请参看  https://yq.aliyun.com/articles/57895
# STS的官方文档请参看  https://help.aliyun.com/document_detail/28627.html

# 首先初始化AccessKeyId、AccessKeySecret、Endpoint等信息。
# 通过环境变量获取，或者把诸如“<你的AccessKeyId>”替换成真实的AccessKeyId等。
# 注意：AccessKeyId、AccessKeySecret为子用户的密钥。
# RoleArn可以在控制台的“访问控制  > 角色管理  > 管理  > 基本信息  > Arn”上查看。
#
# 以杭州区域为例，Endpoint可以是：
#   http://oss-cn-hangzhou.aliyuncs.com
#   https://oss-cn-hangzhou.aliyuncs.com
# 分别以HTTP、HTTPS协议访问。
access_key_id = os.getenv('OSS_TEST_STS_ID', '<你的AccessKeyId>')
access_key_secret = os.getenv('OSS_TEST_STS_KEY', '<你的AccessKeySecret>')
bucket_name = os.getenv('OSS_TEST_BUCKET', '<你的Bucket>')
endpoint = os.getenv('OSS_TEST_ENDPOINT', '<你的访问域名>')
sts_role_arn = os.getenv('OSS_TEST_STS_ARN', '<你的Role Arn>')


# 确认上面的参数都填写正确了
for param in (access_key_id, access_key_secret, bucket_name, endpoint, sts_role_arn):
    assert '<' not in param, '请设置参数：' + param


class StsToken(object):
    """AssumeRole返回的临时用户密钥
    :param str access_key_id: 临时用户的access key id
    :param str access_key_secret: 临时用户的access key secret
    :param int expiration: 过期时间，UNIX时间，自1970年1月1日UTC零点的秒数
    :param str security_token: 临时用户Token
    :param str request_id: 请求ID
    """
    def __init__(self):
        self.access_key_id = ''
        self.access_key_secret = ''
        self.expiration = 0
        self.security_token = ''
        self.request_id = ''


def fetch_sts_token(access_key_id, access_key_secret, role_arn):
    """子用户角色扮演获取临时用户的密钥
    :param access_key_id: 子用户的 access key id
    :param access_key_secret: 子用户的 access key secret
    :param role_arn: STS角色的Arn
    :return StsToken: 临时用户密钥
    """
    clt = client.AcsClient(access_key_id, access_key_secret, 'cn-hangzhou')
    req = AssumeRoleRequest.AssumeRoleRequest()

    req.set_accept_format('json')
    req.set_RoleArn(role_arn)
    req.set_RoleSessionName('oss-python-sdk-example')

    body = clt.do_action_with_exception(req)

    j = json.loads(oss2.to_unicode(body))

    token = StsToken()

    token.access_key_id = j['Credentials']['AccessKeyId']
    token.access_key_secret = j['Credentials']['AccessKeySecret']
    token.security_token = j['Credentials']['SecurityToken']
    token.request_id = j['RequestId']
    token.expiration = oss2.utils.to_unixtime(j['Credentials']['Expiration'], '%Y-%m-%dT%H:%M:%SZ')

    return token


# 创建Bucket对象，所有Object相关的接口都可以通过Bucket对象来进行
token = fetch_sts_token(access_key_id, access_key_secret, sts_role_arn)
auth = oss2.StsAuth(token.access_key_id, token.access_key_secret, token.security_token)
bucket = oss2.Bucket(auth, endpoint, bucket_name)


# 上传一段字符串。Object名是motto.txt，内容是一段名言。
bucket.put_object('motto.txt', 'Never give up. - Jack Ma')


# 下载到本地文件
bucket.get_object_to_file('motto.txt', '本地座右铭.txt')


# 删除名为motto.txt的Object
bucket.delete_object('motto.txt')


# 清除本地文件
os.remove(u'本地座右铭.txt')



# -*- coding: utf-8 -*-

"""
oss2.encryption
~~~~~~~~~~~~~~

该模块包含了客户端加解密相关的函数和类。
"""
import abc
import hashlib
import json
import os
import copy
import logging
import struct
from functools import partial

import six
from Crypto.Cipher import PKCS1_OAEP, PKCS1_v1_5
from Crypto.PublicKey import RSA
from aliyunsdkcore import client
from aliyunsdkcore.acs_exception.exceptions import ServerException, ClientException
from aliyunsdkcore.http import format_type, method_type
from aliyunsdkkms.request.v20160120 import GenerateDataKeyRequest, DecryptRequest, EncryptRequest

from . import models
from . import headers
from . import utils
from .utils import b64decode_from_string, b64encode_as_string
from .compat import to_bytes, to_unicode
from .exceptions import ClientError, OpenApiFormatError, OpenApiServerError

logger = logging.getLogger(__name__)


class EncryptionMaterials(object):
    def __init__(self, desc, key_pair=None, custom_master_key_id=None, passphrase=None):
        self.desc = {}
        if desc:
            if isinstance(desc, dict):
                self.desc = desc
            else:
                raise ClientError('Invalid type, the type of mat_desc must be dict!')
        if key_pair and custom_master_key_id:
            raise ClientError('Both key_pair and custom_master_key_id are not none')

        if key_pair and not isinstance(key_pair, dict):
            raise ClientError('Invalid type, the type of key_pair must be dict!')

        self.key_pair = key_pair
        self.custom_master_key_id = custom_master_key_id
        self.passphrase = passphrase

    def add_description(self, key, value):
        self.desc[key] = value

    def add_descriptions(self, descriptions):
        for key in descriptions:
            self.desc[key] = descriptions[key]


@six.add_metaclass(abc.ABCMeta)
class BaseCryptoProvider(object):
    """CryptoProvider 基类，提供基础的数据加密解密adapter

    """

    def __init__(self, cipher, mat_desc=None):
        if not cipher:
            raise ClientError('Please initialize the value of cipher!')
        self.cipher = cipher
        self.cek_alg = None
        self.wrap_alg = None
        self.mat_desc = None
        self.encryption_materials_dict = {}
        if mat_desc:
            if isinstance(mat_desc, dict):
                self.mat_desc = mat_desc
            else:
                raise ClientError('Invalid type, the type of mat_desc must be dict!')

    @abc.abstractmethod
    def get_key(self):
        pass

    def get_iv(self):
        return self.cipher.get_iv()

    @staticmethod
    def make_encrypt_adapter(stream, cipher):
        return utils.make_cipher_adapter(stream, partial(cipher.encrypt))

    @staticmethod
    def make_decrypt_adapter(stream, cipher, discard=0):
        return utils.make_cipher_adapter(stream, partial(cipher.decrypt), discard)

    @abc.abstractmethod
    def decrypt_encrypted_key(self, encrypted_key):
        pass

    @abc.abstractmethod
    def decrypt_encrypted_iv(self, encrypted_iv):
        pass

    @abc.abstractmethod
    def reset_encryption_materials(self, encryption_materials):
        pass

    def adjust_range(self, start, end):
        return self.cipher.adjust_range(start, end)

    @abc.abstractmethod
    def create_content_material(self):
        pass

    def add_encryption_materials(self, encryption_materials):
        if encryption_materials.desc:
            key = frozenset(encryption_materials.desc.items())
            self.encryption_materials_dict[key] = encryption_materials

    def get_encryption_materials(self, desc):
        if desc:
            key = frozenset(desc.items())
            if key in self.encryption_materials_dict.keys():
                return self.encryption_materials_dict[key]


_LOCAL_RSA_TMP_DIR = '.oss-local-rsa'


@six.add_metaclass(abc.ABCMeta)
class LocalRsaProvider(BaseCryptoProvider):
    """使用本地RSA加密数据密钥。

        :param str dir: 本地RSA公钥私钥存储路径
        :param str key: 本地RSA公钥私钥名称前缀
        :param str passphrase: 本地RSA公钥私钥密码
        :param class cipher: 数据加密，默认aes256，用户可自行实现对称加密算法，需符合AESCipher注释规则
    """

    DEFAULT_PUB_KEY_SUFFIX = '.public_key.pem'
    DEFAULT_PRIV_KEY_SUFFIX = '.private_key.pem'

    def __init__(self, dir=None, key='', passphrase=None, cipher=utils.AESCTRCipher(),
                 pub_key_suffix=DEFAULT_PUB_KEY_SUFFIX, private_key_suffix=DEFAULT_PRIV_KEY_SUFFIX):

        super(LocalRsaProvider, self).__init__(cipher=cipher)

        self.wrap_alg = headers.RSA_NONE_OAEPWithSHA1AndMGF1Padding
        keys_dir = dir or os.path.join(os.path.expanduser('~'), _LOCAL_RSA_TMP_DIR)

        priv_key_path = os.path.join(keys_dir, key + private_key_suffix)
        pub_key_path = os.path.join(keys_dir, key + pub_key_suffix)
        try:
            if os.path.exists(priv_key_path) and os.path.exists(pub_key_path):
                with open(priv_key_path, 'rb') as f:
                    self.__decrypt_obj = PKCS1_OAEP.new(RSA.importKey(f.read(), passphrase=passphrase))

                with open(pub_key_path, 'rb') as f:
                    self.__encrypt_obj = PKCS1_OAEP.new(RSA.importKey(f.read(), passphrase=passphrase))

            else:
                logger.warn('The file path of private key or public key is not exist, will generate key pair')
                private_key = RSA.generate(2048)
                public_key = private_key.publickey()

                self.__encrypt_obj = PKCS1_OAEP.new(public_key)
                self.__decrypt_obj = PKCS1_OAEP.new(private_key)

                utils.makedir_p(keys_dir)
                with open(priv_key_path, 'wb') as f:
                    f.write(private_key.exportKey(passphrase=passphrase))

                with open(pub_key_path, 'wb') as f:
                    f.write(public_key.exportKey(passphrase=passphrase))
        except (ValueError, TypeError, IndexError) as e:
            raise ClientError(str(e))

    def get_key(self):
        return self.cipher.get_key()

    def decrypt_encrypted_key(self, encrypted_key):
        try:
            return self.__decrypt_data(encrypted_key)
        except (TypeError, ValueError) as e:
            raise ClientError(str(e))

    def decrypt_encrypted_iv(self, encrypted_iv):
        try:
            return self.__decrypt_data(encrypted_iv)
        except (TypeError, ValueError) as e:
            raise ClientError(str(e))

    def reset_encryption_materials(self, encryption_materials):
        raise ClientError("do not support reset_encryption_materials!")

    def create_content_material(self):
        plain_key = self.get_key()
        encrypted_key = self.__encrypt_data(plain_key)
        plain_iv = self.get_iv()
        encrypted_iv = self.__encrypt_data(plain_iv)
        cipher = copy.copy(self.cipher)
        wrap_alg = self.wrap_alg
        mat_desc = self.mat_desc

        cipher.initialize(plain_key, plain_iv)

        content_crypto_material = models.ContentCryptoMaterial(cipher, wrap_alg, encrypted_key, encrypted_iv,
                                                               mat_desc)
        return content_crypto_material

    def __encrypt_data(self, data):
        return self.__encrypt_obj.encrypt(data)

    def __decrypt_data(self, data):
        return self.__decrypt_obj.decrypt(data)


@six.add_metaclass(abc.ABCMeta)
class RsaProvider(BaseCryptoProvider):
    """使用本地RSA加密数据密钥。

        :param str dir: 本地RSA公钥私钥存储路径
        :param str key: 本地RSA公钥私钥名称前缀
        :param str passphrase: 本地RSA公钥私钥密码
        :param class cipher: 数据加密，默认aes256，用户可自行实现对称加密算法，需符合AESCipher注释规则
    """

    def __init__(self, key_pair, passphrase=None, cipher=utils.AESCTRCipher(), mat_desc=None):

        super(RsaProvider, self).__init__(cipher=cipher, mat_desc=mat_desc)
        self.wrap_alg = headers.RSA_NONE_PKCS1Padding_WRAP_ALGORITHM

        if key_pair and not isinstance(key_pair, dict):
            raise ClientError('Invalid type, the type of key_pair must be dict!')

        try:
            if 'public_key' in key_pair:
                self.__encrypt_obj = PKCS1_v1_5.new(RSA.importKey(key_pair['public_key'], passphrase=passphrase))

            if 'private_key' in key_pair:
                self.__decrypt_obj = PKCS1_v1_5.new(RSA.importKey(key_pair['private_key'], passphrase=passphrase))
        except (ValueError, TypeError) as e:
            raise ClientError(str(e))

    def get_key(self):
        return self.cipher.get_key()

    def decrypt_encrypted_key(self, encrypted_key):
        try:
            return self.__decrypt_data(encrypted_key)
        except (TypeError, ValueError) as e:
            raise ClientError(str(e))

    def decrypt_encrypted_iv(self, encrypted_iv):
        try:
            return self.__decrypt_data(encrypted_iv)
        except (TypeError, ValueError) as e:
            raise ClientError(str(e))

    def reset_encryption_materials(self, encryption_materials):
        return RsaProvider(encryption_materials.key_pair, encryption_materials.passphrase, self.cipher,
                           encryption_materials.desc)

    def create_content_material(self):
        plain_key = self.get_key()
        encrypted_key = self.__encrypt_data(plain_key)
        plain_iv = self.get_iv()
        encrypted_iv = self.__encrypt_data(plain_iv)
        cipher = copy.copy(self.cipher)
        wrap_alg = self.wrap_alg
        mat_desc = self.mat_desc

        cipher.initialize(plain_key, plain_iv)

        content_crypto_material = models.ContentCryptoMaterial(cipher, wrap_alg, encrypted_key, encrypted_iv,
                                                               mat_desc)
        return content_crypto_material

    def __encrypt_data(self, data):
        return self.__encrypt_obj.encrypt(data)

    def __decrypt_data(self, data):
        decrypted_data = self.__decrypt_obj.decrypt(data, object)
        if decrypted_data == object:
            raise ClientError('Decrypted data error, please check you key pair!')
        return decrypted_data


class AliKMSProvider(BaseCryptoProvider):
    """使用aliyun kms服务加密数据密钥。kms的详细说明参见
        https://help.aliyun.com/product/28933.html?spm=a2c4g.11186623.3.1.jlYT4v
        此接口在py3.3下暂时不可用，详见
        https://github.com/aliyun/aliyun-openapi-python-sdk/issues/61

        :param str access_key_id: 可以访问kms密钥服务的access_key_id
        :param str access_key_secret: 可以访问kms密钥服务的access_key_secret
        :param str region: kms密钥服务地区
        :param str cmkey: 用户主密钥
        :param str sts_token: security token，如果使用的是临时AK需提供
        :param str passphrase: kms密钥服务密码
        :param class cipher: 数据加密，默认aes256，当前仅支持默认实现
    """

    def __init__(self, access_key_id, access_key_secret, region, cmk_id, sts_token=None, passphrase=None,
                 cipher=utils.AESCTRCipher(), mat_desc=None):

        super(AliKMSProvider, self).__init__(cipher=cipher, mat_desc=mat_desc)
        if not isinstance(cipher, utils.AESCTRCipher):
            raise ClientError('AliKMSProvider only support AES256 cipher now')
        self.wrap_alg = headers.KMS_ALI_WRAP_ALGORITHM
        self.custom_master_key_id = cmk_id
        self.sts_token = sts_token
        self.context = '{"x-passphrase":"' + passphrase + '"}' if passphrase else ''
        self.kms_client = client.AcsClient(access_key_id, access_key_secret, region)

    def get_key(self):
        plain_key, encrypted_key = self.__generate_data_key()
        return plain_key, encrypted_key

    def decrypt_encrypted_key(self, encrypted_key):
        return b64decode_from_string(self.__decrypt_data(encrypted_key))

    def decrypt_encrypted_iv(self, encrypted_iv, deprecated=False):
        if deprecated:
            return self.__decrypt_data(encrypted_iv)
        return b64decode_from_string(self.__decrypt_data(encrypted_iv))

    def reset_encryption_materials(self, encryption_materials):
        provider = copy.copy(self)
        provider.custom_master_key_id = encryption_materials.custom_master_key_id
        provider.context = '{"x-passphrase":"' + encryption_materials.passphrase + '"}' if encryption_materials.passphrase else ''
        provider.mat_desc = encryption_materials.desc
        return provider

    def create_content_material(self):
        plain_key, encrypted_key = self.get_key()
        plain_iv = self.get_iv()
        encrypted_iv = self.__encrypt_data(b64encode_as_string(plain_iv))
        cipher = copy.copy(self.cipher)
        wrap_alg = self.wrap_alg
        mat_desc = self.mat_desc

        cipher.initialize(plain_key, plain_iv)

        content_crypto_material = models.ContentCryptoMaterial(cipher, wrap_alg, encrypted_key, encrypted_iv,
                                                               mat_desc)
        return content_crypto_material

    def __generate_data_key(self):
        req = GenerateDataKeyRequest.GenerateDataKeyRequest()

        req.set_accept_format(format_type.JSON)
        req.set_method(method_type.POST)

        req.set_KeyId(self.custom_master_key_id)
        req.set_KeySpec('AES_256')
        req.set_NumberOfBytes(32)
        req.set_EncryptionContext(self.context)
        if self.sts_token:
            req.set_STSToken(self.sts_token)

        resp = self.__do(req)

        return b64decode_from_string(resp['Plaintext']), resp['CiphertextBlob']

    def __encrypt_data(self, data):
        req = EncryptRequest.EncryptRequest()

        req.set_accept_format(format_type.JSON)
        req.set_method(method_type.POST)
        req.set_KeyId(self.custom_master_key_id)
        req.set_Plaintext(data)
        req.set_EncryptionContext(self.context)
        if self.sts_token:
            req.set_STSToken(self.sts_token)

        resp = self.__do(req)

        return resp['CiphertextBlob']

    def __decrypt_data(self, data):
        req = DecryptRequest.DecryptRequest()

        req.set_accept_format(format_type.JSON)
        req.set_method(method_type.POST)
        req.set_CiphertextBlob(data)
        req.set_EncryptionContext(self.context)
        if self.sts_token:
            req.set_STSToken(self.sts_token)

        resp = self.__do(req)
        return resp['Plaintext']

    def __do(self, req):

        try:
            body = self.kms_client.do_action_with_exception(req)
            return json.loads(to_unicode(body))
        except ServerException as e:
            raise OpenApiServerError(e.http_status, e.request_id, e.message, e.error_code)
        except ClientException as e:
            raise ClientError(e.message)
        except (KeyError, ValueError, TypeError) as e:
            raise OpenApiFormatError('Json Error: ' + str(e))



from .common import *


class TestBucketTransferAccelerat(OssTestCase):
    def test_bucket_transfer_acceleration_normal(self):
        result = self.bucket.put_bucket_transfer_acceleration('true')
        self.assertEqual(200, result.status)

        get_result = self.bucket.get_bucket_transfer_acceleration()
        self.assertEqual('true', get_result.enabled)

    def test_bucket_worm_illegal(self):
        self.assertRaises(oss2.exceptions.NoSuchTransferAccelerationConfiguration, self.bucket.get_bucket_transfer_acceleration)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

"""
oss2.utils
----------

工具函数模块。
"""

from email.utils import formatdate

import logging
import os.path
import mimetypes
import socket
import hashlib
import base64
import threading
import calendar
import datetime
import time
import errno

import binascii
import crcmod
import re
import sys
import random
import abc, six
import struct

from Crypto.Cipher import AES
from Crypto import Random
from Crypto.Util import Counter

from .crc64_combine import mkCombineFun
from .compat import to_string, to_bytes, urlparse
from .exceptions import ClientError, InconsistentError, RequestError, OpenApiFormatError
from . import defaults

logger = logging.getLogger(__name__)

_EXTRA_TYPES_MAP = {
    ".js": "application/javascript",
    ".xlsx": "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
    ".xltx": "application/vnd.openxmlformats-officedocument.spreadsheetml.template",
    ".potx": "application/vnd.openxmlformats-officedocument.presentationml.template",
    ".ppsx": "application/vnd.openxmlformats-officedocument.presentationml.slideshow",
    ".pptx": "application/vnd.openxmlformats-officedocument.presentationml.presentation",
    ".sldx": "application/vnd.openxmlformats-officedocument.presentationml.slide",
    ".docx": "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
    ".dotx": "application/vnd.openxmlformats-officedocument.wordprocessingml.template",
    ".xlam": "application/vnd.ms-excel.addin.macroEnabled.12",
    ".xlsb": "application/vnd.ms-excel.sheet.binary.macroEnabled.12",
    ".apk": "application/vnd.android.package-archive"
}


def b64encode_as_string(data):
    return to_string(base64.b64encode(to_bytes(data)))


def b64decode_from_string(data):
    try:
        return base64.b64decode(to_string(data))
    except (TypeError, binascii.Error) as e:
        raise OpenApiFormatError('Base64 Error: ' + to_string(data))


def content_md5(data):
    """计算data的MD5值，经过Base64编码并返回str类型。

    返回值可以直接作为HTTP Content-Type头部的值
    """
    m = hashlib.md5(to_bytes(data))
    return b64encode_as_string(m.digest())


def md5_string(data):
    """返回 `data` 的MD5值，以十六进制可读字符串（32个小写字符）的方式。"""
    return hashlib.md5(to_bytes(data)).hexdigest()


def content_type_by_name(name):
    """根据文件名，返回Content-Type。"""
    ext = os.path.splitext(name)[1].lower()
    if ext in _EXTRA_TYPES_MAP:
        return _EXTRA_TYPES_MAP[ext]

    return mimetypes.guess_type(name)[0]


def set_content_type(headers, name):
    """根据文件名在headers里设置Content-Type。如果headers中已经存在Content-Type，则直接返回。"""
    headers = headers or {}

    if 'Content-Type' in headers:
        return headers

    content_type = content_type_by_name(name)
    if content_type:
        headers['Content-Type'] = content_type

    return headers


def is_ip_or_localhost(netloc):
    """判断网络地址是否为IP或localhost。"""
    is_ipv6 = False
    right_bracket_index = netloc.find(']')
    if netloc[0] == '[' and right_bracket_index > 0:
        loc = netloc[1:right_bracket_index]
        is_ipv6 = True
    else:
        loc = netloc.split(':')[0]

    if loc == 'localhost':
        return True

    try:
        if is_ipv6:
            socket.inet_pton(socket.AF_INET6, loc)  # IPv6
        else:
            socket.inet_aton(loc)  # Only IPv4
    except socket.error:
        return False

    return True


_ALPHA_NUM = 'abcdefghijklmnopqrstuvwxyz0123456789'
_HYPHEN = '-'
_BUCKET_NAME_CHARS = set(_ALPHA_NUM + _HYPHEN)
_MAX_UINT32 = 2 ** 32 - 1
_MAX_UINT64 = 2 ** 64 - 1


def is_valid_bucket_name(name):
    """判断是否为合法的Bucket名"""
    if len(name) < 3 or len(name) > 63:
        return False

    if name[-1] == _HYPHEN:
        return False

    if name[0] not in _ALPHA_NUM:
        return False

    return set(name) <= _BUCKET_NAME_CHARS

def is_valid_endpoint(endpoint):
    """判断是否为合法的endpoint"""
    if endpoint is None:
        return False

    pattern = '^([a-zA-Z]+://)?[\w.-]+(:\d+)?$'
    if re.match(pattern, endpoint):
        return True

    return False

def change_endianness_if_needed(bytes_array):
    if sys.byteorder == 'little':
        bytes_array.reverse();


class SizedFileAdapter(object):
    """通过这个适配器（Adapter），可以把原先的 `file_object` 的长度限制到等于 `size`。"""
    def __init__(self, file_object, size):
        self.file_object = file_object
        self.size = size
        self.offset = 0

    def read(self, amt=None):
        if self.offset >= self.size:
            return ''

        if (amt is None or amt < 0) or (amt + self.offset >= self.size):
            data = self.file_object.read(self.size - self.offset)
            self.offset = self.size
            return data

        self.offset += amt
        return self.file_object.read(amt)

    @property
    def len(self):
        return self.size


def how_many(m, n):
    return (m + n - 1) // n


def file_object_remaining_bytes(fileobj):
    current = fileobj.tell()

    fileobj.seek(0, os.SEEK_END)
    end = fileobj.tell()
    fileobj.seek(current, os.SEEK_SET)

    return end - current


def _has_data_size_attr(data):
    return hasattr(data, '__len__') or hasattr(data, 'len') or (hasattr(data, 'seek') and hasattr(data, 'tell'))


def _get_data_size(data):
    if hasattr(data, '__len__'):
        return len(data)

    if hasattr(data, 'len'):
        return data.len

    if hasattr(data, 'seek') and hasattr(data, 'tell'):
        return file_object_remaining_bytes(data)

    return None


_CHUNK_SIZE = 8 * 1024


def make_progress_adapter(data, progress_callback, size=None):
    """返回一个适配器，从而在读取 `data` ，即调用read或者对其进行迭代的时候，能够
     调用进度回调函数。当 `size` 没有指定，且无法确定时，上传回调函数返回的总字节数为None。

    :param data: 可以是bytes、file object或iterable
    :param progress_callback: 进度回调函数，参见 :ref:`progress_callback`
    :param size: 指定 `data` 的大小，可选

    :return: 能够调用进度回调函数的适配器
    """
    data = to_bytes(data)

    if size is None:
        size = _get_data_size(data)

    if size is None:
        if hasattr(data, 'read'):
            return _FileLikeAdapter(data, progress_callback)
        elif hasattr(data, '__iter__'):
            return _IterableAdapter(data, progress_callback)
        else:
            raise ClientError('{0} is not a file object, nor an iterator'.format(data.__class__.__name__))
    else:
        return _BytesAndFileAdapter(data, progress_callback, size)


def make_crc_adapter(data, init_crc=0, discard=0):
    """返回一个适配器，从而在读取 `data` ，即调用read或者对其进行迭代的时候，能够计算CRC。

    :param discard:
    :return:
    :param data: 可以是bytes、file object或iterable
    :param init_crc: 初始CRC值，可选

    :return: 能够调用计算CRC函数的适配器
    """
    data = to_bytes(data)

    # bytes or file object
    if _has_data_size_attr(data):
        if discard:
            raise ClientError('Bytes of file object adapter does not support discard bytes')
        return _BytesAndFileAdapter(data, size=_get_data_size(data), crc_callback=Crc64(init_crc))
    # file-like object
    elif hasattr(data, 'read'):
        return _FileLikeAdapter(data, crc_callback=Crc64(init_crc), discard=discard)
    # iterator
    elif hasattr(data, '__iter__'):
        if discard:
            raise ClientError('Iterator adapter does not support discard bytes')
        return _IterableAdapter(data, crc_callback=Crc64(init_crc))
    else:
        raise ClientError('{0} is not a file object, nor an iterator'.format(data.__class__.__name__))


def calc_obj_crc_from_parts(parts, init_crc=0):
    object_crc = 0
    crc_obj = Crc64(init_crc)
    for part in parts:
        if not part.part_crc or not part.size:
            return None
        else:
            object_crc = crc_obj.combine(object_crc, part.part_crc, part.size)
    return object_crc


def make_cipher_adapter(data, cipher_callback, discard=0):
    """返回一个适配器，从而在读取 `data` ，即调用read或者对其进行迭代的时候，能够进行加解密操作。

        :param encrypt:
        :param cipher_callback:
        :param discard: 读取时需要丢弃的字节
        :param data: 可以是bytes、file object或iterable

        :return: 能够客户端加密函数的适配器
        """
    data = to_bytes(data)

    # bytes or file object
    if _has_data_size_attr(data):
        if discard:
            raise ClientError('Bytes of file object adapter does not support discard bytes')
        return _BytesAndFileAdapter(data, size=_get_data_size(data), cipher_callback=cipher_callback)
    if hasattr(data, 'read'):
        return _FileLikeAdapter(data, cipher_callback=cipher_callback, discard=discard)
    # iterator
    elif hasattr(data, '__iter__'):
        if discard:
            raise ClientError('Iterator adapter does not support discard bytes')
        return _IterableAdapter(data, cipher_callback=cipher_callback)
    else:
        raise ClientError('{0} is not a file object'.format(data.__class__.__name__))


def check_crc(operation, client_crc, oss_crc, request_id):
    if client_crc is not None and oss_crc is not None and client_crc != oss_crc:
        e = InconsistentError("InconsistentError: req_id: {0}, operation: {1}, CRC checksum of client: {2} is mismatch "
                              "with oss: {3}".format(request_id, operation, client_crc, oss_crc))
        logger.error("Exception: {0}".format(e))
        raise e


def _invoke_crc_callback(crc_callback, content, discard=0):
    if crc_callback:
        crc_callback(content[discard:])


def _invoke_progress_callback(progress_callback, consumed_bytes, total_bytes):
    if progress_callback:
        progress_callback(consumed_bytes, total_bytes)


def _invoke_cipher_callback(cipher_callback, content, discard=0):
    if cipher_callback:
        content = cipher_callback(content)
        return content[discard:]
    return content


class _IterableAdapter(object):
    def __init__(self, data, progress_callback=None, crc_callback=None, cipher_callback=None):
        self.iter = iter(data)
        self.progress_callback = progress_callback
        self.offset = 0
        
        self.crc_callback = crc_callback
        self.cipher_callback = cipher_callback

    def __iter__(self):
        return self

    def __next__(self):
        return self.next()

    def next(self):            
        _invoke_progress_callback(self.progress_callback, self.offset, None)

        content = next(self.iter)
        self.offset += len(content)
                
        _invoke_crc_callback(self.crc_callback, content)

        content = _invoke_cipher_callback(self.cipher_callback, content)

        return content
    
    @property
    def crc(self):
        if self.crc_callback:
            return self.crc_callback.crc
        elif self.iter:
            return self.iter.crc
        else:
            return None


class _FileLikeAdapter(object):
    """通过这个适配器，可以给无法确定内容长度的 `fileobj` 加上进度监控。

    :param fileobj: file-like object，只要支持read即可
    :param progress_callback: 进度回调函数
    """

    def __init__(self, fileobj, progress_callback=None, crc_callback=None, cipher_callback=None, discard=0):
        self.fileobj = fileobj
        self.progress_callback = progress_callback
        self.offset = 0

        self.crc_callback = crc_callback
        self.cipher_callback = cipher_callback
        self.discard = discard
        self.read_all = False

    def __iter__(self):
        return self

    def __next__(self):
        return self.next()

    def next(self):
        if self.read_all:
            raise StopIteration

        content = self.read(_CHUNK_SIZE)

        if content:
            return content
        else:
            raise StopIteration

    def read(self, amt=None):
        offset_start = self.offset
        if offset_start < self.discard and amt and self.cipher_callback:
            amt += self.discard

        content = self.fileobj.read(amt)
        if not content:
            self.read_all = True
            _invoke_progress_callback(self.progress_callback, self.offset, None)
        else:
            _invoke_progress_callback(self.progress_callback, self.offset, None)

            self.offset += len(content)

            real_discard = 0
            if offset_start < self.discard:
                if len(content) <= self.discard:
                    real_discard = len(content)
                else:
                    real_discard = self.discard

            _invoke_crc_callback(self.crc_callback, content, real_discard)
            content = _invoke_cipher_callback(self.cipher_callback, content, real_discard)

            self.discard -= real_discard
        return content

    @property
    def crc(self):
        if self.crc_callback:
            return self.crc_callback.crc
        elif self.fileobj:
            return self.fileobj.crc
        else:
            return None


class _BytesAndFileAdapter(object):
    """通过这个适配器，可以给 `data` 加上进度监控。

    :param data: 可以是unicode字符串（内部会转换为UTF-8编码的bytes）、bytes或file object
    :param progress_callback: 用户提供的进度报告回调，形如 callback(bytes_read, total_bytes)。
        其中bytes_read是已经读取的字节数；total_bytes是总的字节数。
    :param int size: `data` 包含的字节数。
    """
    def __init__(self, data, progress_callback=None, size=None, crc_callback=None, cipher_callback=None):
        self.data = to_bytes(data)
        self.progress_callback = progress_callback
        self.size = size
        self.offset = 0
        
        self.crc_callback = crc_callback
        self.cipher_callback = cipher_callback

    @property
    def len(self):
        return self.size

    # for python 2.x
    def __bool__(self):
        return True
    # for python 3.x
    __nonzero__=__bool__

    def __iter__(self):
        return self

    def __next__(self):
        return self.next()

    def next(self):
        content = self.read(_CHUNK_SIZE)

        if content:
            return content
        else:
            raise StopIteration

    def read(self, amt=None):
        if self.offset >= self.size:
            return to_bytes('')

        if amt is None or amt < 0:
            bytes_to_read = self.size - self.offset
        else:
            bytes_to_read = min(amt, self.size - self.offset)

        if isinstance(self.data, bytes):
            content = self.data[self.offset:self.offset+bytes_to_read]
        else:
            content = self.data.read(bytes_to_read)

        self.offset += bytes_to_read
            
        _invoke_progress_callback(self.progress_callback, min(self.offset, self.size), self.size)

        _invoke_crc_callback(self.crc_callback, content)

        content = _invoke_cipher_callback(self.cipher_callback, content)

        return content
    
    @property
    def crc(self):
        if self.crc_callback:
            return self.crc_callback.crc
        elif self.data:
            return self.data.crc
        else:
            return None


class Crc64(object):

    _POLY = 0x142F0E1EBA9EA3693
    _XOROUT = 0XFFFFFFFFFFFFFFFF
    
    def __init__(self, init_crc=0):
        self.crc64 = crcmod.Crc(self._POLY, initCrc=init_crc, rev=True, xorOut=self._XOROUT)

        self.crc64_combineFun = mkCombineFun(self._POLY, initCrc=init_crc, rev=True, xorOut=self._XOROUT)

    def __call__(self, data):
        self.update(data)
    
    def update(self, data):
        self.crc64.update(data)

    def combine(self, crc1, crc2, len2):
        return self.crc64_combineFun(crc1, crc2, len2)
    
    @property
    def crc(self):
        return self.crc64.crcValue

class Crc32(object):
    _POLY = 0x104C11DB7
    _XOROUT = 0xFFFFFFFF
    
    def __init__(self, init_crc=0):
        self.crc32 = crcmod.Crc(self._POLY, initCrc=init_crc, rev=True, xorOut=self._XOROUT)

    def __call__(self, data):
        self.update(data)
    
    def update(self, data):
        self.crc32.update(data)
    
    @property
    def crc(self):
        return self.crc32.crcValue


_AES_256_KEY_SIZE = 32
_AES_BLOCK_LEN = 16
_AES_BLOCK_BITS_LEN = 8 * 16

AES_GCM = 'AES/GCM/NoPadding'
AES_CTR = 'AES/CTR/NoPadding'


@six.add_metaclass(abc.ABCMeta)
class AESCipher(object):
    """AES256 加密实现。
            :param str key: 对称加密数据密钥
            :param str start: 对称加密初始随机值
        .. note::
            用户可自行实现对称加密算法，需服务如下规则：
            1、提供对称加密算法名，ALGORITHM
            2、提供静态方法，返回加密密钥和初始随机值（若算法不需要初始随机值，也需要提供）
            3、提供加密解密方法
    """

    # aes 256, key always is 32 bytes
    def __init__(self):
        self.alg = None
        self.key_len = _AES_256_KEY_SIZE
        self.block_size_len = _AES_BLOCK_LEN
        self.block_size_len_in_bits = _AES_BLOCK_BITS_LEN

    @abc.abstractmethod
    def get_key(self):
        pass

    @abc.abstractmethod
    def get_iv(self):
        pass

    @abc.abstractmethod
    def initialize(self, key, iv, off=0):
        pass

    @abc.abstractmethod
    def encrypt(self, raw):
        pass

    @abc.abstractmethod
    def decrypt(self, enc):
        pass

    @abc.abstractmethod
    def determine_part_size(self, data_size, excepted_part_size=None):
        pass

    def adjust_range(self, start, end):
        return start, end

    def is_block_aligned(self, offset):
        if offset is None:
            offset = 0
        return 0 == offset % self.block_size_len

    def is_valid_part_size(self, part_size, data_size=None):
        return True


class AESCTRCipher(AESCipher):
    """AES256 加密实现。
        :param str key: 对称加密数据密钥
        :param str start: 对称加密初始随机值
    .. note::
        用户可自行实现对称加密算法，需服务如下规则：
        1、提供对称加密算法名，ALGORITHM
        2、提供静态方法，返回加密密钥和初始随机值（若算法不需要初始随机值，也需要提供）
        3、提供加密解密方法
    """

    def __init__(self):
        super(AESCTRCipher, self).__init__()
        self.alg = AES_CTR
        self.__cipher = None

    def get_key(self):
        return random_key(self.key_len)

    def get_iv(self):
        return random_iv()

    def initialize(self, key, iv, offset=0):
        counter = iv_to_big_int(iv) + offset
        self.initial_by_counter(key, counter)

    def initial_by_counter(self, key, counter):
        ctr = Counter.new(self.block_size_len_in_bits, initial_value=counter)
        self.__cipher = AES.new(key, AES.MODE_CTR, counter=ctr)

    def encrypt(self, raw):
        return self.__cipher.encrypt(raw)

    def decrypt(self, enc):
        return self.__cipher.encrypt(enc)

    def adjust_range(self, start, end):
        if start:
            if end:
                if start <= end:
                    start = (start // self.block_size_len) * self.block_size_len
            else:
                start = (start // self.block_size_len) * self.block_size_len
        return start, end

    def is_valid_part_size(self, part_size, data_size):
        if not self.is_block_aligned(part_size) or part_size < defaults.min_part_size:
            return False

        if part_size * defaults.max_part_count < data_size:
            return False
        return True

    def calc_offset(self, offset):
        if not self.is_block_aligned(offset):
            raise ClientError('offset is not align to encrypt block')
        return offset // self.block_size_len

    def determine_part_size(self, data_size, excepted_part_size=None):
        if excepted_part_size:
            if self.is_valid_part_size(excepted_part_size, data_size):
                return excepted_part_size
            # excepted_part_size is not aligned
            elif excepted_part_size * defaults.max_part_count >= data_size:
                part_size = int(excepted_part_size / self.block_size_len + 1) * self.block_size_len
                return part_size

        # if excepted_part_size is None or is too small, calculate a correct part_size
        part_size = defaults.part_size
        while part_size * defaults.max_part_count < data_size:
            part_size = part_size * 2

        if not self.is_block_aligned(part_size):
            part_size = int(part_size / self.block_size_len + 1) * self.block_size_len

        return part_size


def random_key(key_len):
    return Random.new().read(key_len)


def random_iv():
    iv = Random.new().read(16)
    safe_iv = iv[0:8] + struct.pack(">L", 0) + iv[12:]
    return safe_iv


def iv_to_big_int(iv):
    iv_high_low_pair = struct.unpack(">QQ", iv)
    iv_big_int = iv_high_low_pair[0] << 64 | iv_high_low_pair[1]
    return iv_big_int


_STRPTIME_LOCK = threading.Lock()

_ISO8601_FORMAT = "%Y-%m-%dT%H:%M:%S.000Z"

# A regex to match HTTP Last-Modified header, whose format is 'Sat, 05 Dec 2015 11:10:29 GMT'.
# Its strftime/strptime format is '%a, %d %b %Y %H:%M:%S GMT'

_HTTP_GMT_RE = re.compile(
    r'(?:Mon|Tue|Wed|Thu|Fri|Sat|Sun), (?P<day>0[1-9]|([1-2]\d)|(3[0-1])) (?P<month>Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec) (?P<year>\d+) (?P<hour>([0-1]\d)|(2[0-3])):(?P<minute>[0-5]\d):(?P<second>[0-5]\d) GMT$'
)

_ISO8601_RE = re.compile(
    r'(?P<year>\d+)-(?P<month>01|02|03|04|05|06|07|08|09|10|11|12)-(?P<day>0[1-9]|([1-2]\d)|(3[0-1]))T(?P<hour>([0-1]\d)|(2[0-3])):(?P<minute>[0-5]\d):(?P<second>[0-5]\d)\.000Z$'
)

_MONTH_MAPPING = {
    'Jan': 1,
    'Feb': 2,
    'Mar': 3,
    'Apr': 4,
    'May': 5,
    'Jun': 6,
    'Jul': 7,
    'Aug': 8,
    'Sep': 9,
    'Oct': 10,
    'Nov': 11,
    'Dec': 12
}


def to_unixtime(time_string, format_string):
    with _STRPTIME_LOCK:
        return int(calendar.timegm(time.strptime(time_string, format_string)))


def http_date(timeval=None):
    """返回符合HTTP标准的GMT时间字符串，用strftime的格式表示就是"%a, %d %b %Y %H:%M:%S GMT"。
    但不能使用strftime，因为strftime的结果是和locale相关的。
    """
    return formatdate(timeval, usegmt=True)


def http_to_unixtime(time_string):
    """把HTTP Date格式的字符串转换为UNIX时间（自1970年1月1日UTC零点的秒数）。

    HTTP Date形如 `Sat, 05 Dec 2015 11:10:29 GMT` 。
    """
    m = _HTTP_GMT_RE.match(time_string)

    if not m:
        raise ValueError(time_string + " is not in valid HTTP date format")

    day = int(m.group('day'))
    month = _MONTH_MAPPING[m.group('month')]
    year = int(m.group('year'))
    hour = int(m.group('hour'))
    minute = int(m.group('minute'))
    second = int(m.group('second'))

    tm = datetime.datetime(year, month, day, hour, minute, second).timetuple()

    return calendar.timegm(tm)


def iso8601_to_unixtime(time_string):
    """把ISO8601时间字符串（形如，2012-02-24T06:07:48.000Z）转换为UNIX时间，精确到秒。"""

    m = _ISO8601_RE.match(time_string)

    if not m:
        raise ValueError(time_string + " is not in valid ISO8601 format")

    day = int(m.group('day'))
    month = int(m.group('month'))
    year = int(m.group('year'))
    hour = int(m.group('hour'))
    minute = int(m.group('minute'))
    second = int(m.group('second'))

    tm = datetime.datetime(year, month, day, hour, minute, second).timetuple()

    return calendar.timegm(tm)


def date_to_iso8601(d):
    return d.strftime(_ISO8601_FORMAT)  # It's OK to use strftime, since _ISO8601_FORMAT is not locale dependent


def iso8601_to_date(time_string):
    timestamp = iso8601_to_unixtime(time_string)
    return datetime.date.fromtimestamp(timestamp)


def makedir_p(dirpath):
    try:
        os.makedirs(dirpath)
    except os.error as e:
        if e.errno != errno.EEXIST:
            raise


def silently_remove(filename):
    """删除文件，如果文件不存在也不报错。"""
    try:
        os.remove(filename)
    except OSError as e:
        if e.errno != errno.ENOENT:
            raise


def force_rename(src, dst):
    try:
        os.rename(src, dst)
    except OSError as e:
        if e.errno == errno.EEXIST:
            silently_remove(dst)
            os.rename(src, dst)
        else:
            raise


def copyfileobj_and_verify(fsrc, fdst, expected_len,
                           chunk_size=16*1024,
                           request_id=''):
    """copy data from file-like object fsrc to file-like object fdst, and verify length"""

    num_read = 0

    while 1:
        buf = fsrc.read(chunk_size)
        if not buf:
            break

        num_read += len(buf)
        fdst.write(buf)

    if num_read != expected_len:
        raise InconsistentError("IncompleteRead from source", request_id)

def _make_line_range_string(range):
    if range is None:
        return ''

    start = range[0]
    last = range[1]

    if start is None and last is None:
        return ''

    return 'line-range=' + _range_internal(start, last)

def _make_split_range_string(range):
    if range is None:
        return ''

    start = range[0]
    last = range[1]

    if start is None and last is None:
        return ''

    return 'split-range=' + _range_internal(start, last)

def _range_internal(start, last):
    def to_str(pos):
        if pos is None:
            return ''
        else:
            return str(pos)

    return to_str(start) + '-' + to_str(last)



# -*- coding: utf-8 -*-

from .common import *
import oss2
from oss2 import models


class TestCryptoMultipart(OssTestCase):
    def test_crypto_init_multipart_with_out_data_size(self):
        crypto_bucket = random.choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key()

        # upload_context is none
        self.assertRaises(oss2.exceptions.ClientError, crypto_bucket.init_multipart_upload, key)

        # data_size is none
        part_size = 1024 * 100
        context = models.MultipartUploadCryptoContext(part_size=part_size)
        self.assertRaises(oss2.exceptions.ClientError, crypto_bucket.init_multipart_upload, key, upload_context=context)

    def test_crypto_init_multipart_invalid_part_size(self):
        crypto_bucket = random.choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key()

        data_size = 1024 * 100
        part_size = random.randint(1, 15) + 1024 * 100
        context = models.MultipartUploadCryptoContext(data_size, part_size)

        # not align to block_size
        self.assertRaises(oss2.exceptions.ClientError, crypto_bucket.init_multipart_upload, key, upload_context=context)

        # part size is small than 100*1024
        part_size = random.randint(1, 1024 * 100 - 1)
        context.part_size = part_size
        self.assertRaises(oss2.exceptions.ClientError, crypto_bucket.init_multipart_upload, key, upload_context=context)

    # 测试不指定part_size的情况，由接口指定part_size
    def test_crypto_init_multipart_with_out_part_size(self):
        crypto_bucket = random.choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key()
        data_size = 1024 * 100

        # init multipart without part_size
        context = models.MultipartUploadCryptoContext(data_size)
        init_result = crypto_bucket.init_multipart_upload(key, upload_context=context)
        self.assertTrue(init_result.status == 200)
        upload_id = init_result.upload_id
        abort_result = crypto_bucket.abort_multipart_upload(key, upload_id)
        self.assertTrue(abort_result.status == 204)

    def test_crypto_abort_multipart(self):
        crypto_bucket = random.choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key()
        content = random_bytes(100 * 1024)

        data_size = 1024 * 100
        part_size = 1024 * 100

        context = models.MultipartUploadCryptoContext(data_size, part_size)
        init_result = crypto_bucket.init_multipart_upload(key, upload_context=context)
        self.assertTrue(init_result.status == 200)
        upload_id = init_result.upload_id

        upload_result = crypto_bucket.upload_part(key, upload_id, 1, content, upload_context=context)
        self.assertTrue(upload_result.status == 200)
        self.assertTrue(upload_result.crc is not None)

        abort_result = crypto_bucket.abort_multipart_upload(key, upload_id)
        self.assertTrue(abort_result.status == 204)

    def test_crypto_multipart_upload_basic(self):
        for upload_contexts_flag in [False, False]:
            crypto_bucket = random.choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
            key = self.random_key()
            content_1 = random_bytes(200 * 1024)
            content_2 = random_bytes(200 * 1024)
            content_3 = random_bytes(100 * 1024)
            content = [content_1, content_2, content_3]
            do_md5 = random.choice((True, False))

            parts = []
            data_size = 1024 * 500
            part_size = 1024 * 200

            context = models.MultipartUploadCryptoContext(data_size, part_size)
            init_result = crypto_bucket.init_multipart_upload(key, upload_context=context)
            self.assertTrue(init_result.status == 200)
            upload_id = init_result.upload_id

            for i in range(3):
                if do_md5:
                    headers = {'Content-Md5': oss2.utils.content_md5(content[i])}
                else:
                    headers = None
                upload_result = crypto_bucket.upload_part(key, upload_id, i + 1, content[i], headers=headers,
                                                          upload_context=context)
                parts.append(oss2.models.PartInfo(i + 1, upload_result.etag, size=len(content[i]),
                                                  part_crc=upload_result.crc))
                self.assertTrue(upload_result.status == 200)
                self.assertTrue(upload_result.crc is not None)

            complete_result = crypto_bucket.complete_multipart_upload(key, upload_id, parts)
            self.assertTrue(complete_result.status == 200)

            get_result_range_1 = crypto_bucket.get_object(key, byte_range=(0, 204799))
            self.assertTrue(get_result_range_1.status == 206)
            content_got_1 = get_result_range_1.read()
            self.assertEqual(content_1, content_got_1)

            get_result_range_2 = crypto_bucket.get_object(key, byte_range=(204800, 409599))
            self.assertTrue(get_result_range_2.status == 206)
            content_got_2 = get_result_range_2.read()
            self.assertEqual(content_2, content_got_2)

            get_result_range_3 = crypto_bucket.get_object(key, byte_range=(409600, 511999))
            self.assertTrue(get_result_range_3.status == 206)
            content_got_3 = get_result_range_3.read()
            self.assertEqual(content_3, content_got_3)

            get_result = crypto_bucket.get_object(key)
            self.assertTrue(get_result.status == 200)
            content_got = get_result.read()
            self.assertEqual(content_1, content_got[0:204800])
            self.assertEqual(content_2, content_got[204800:409600])
            self.assertEqual(content_3, content_got[409600:512000])

    def test_upload_part_copy_from_crypto_source(self):
        crypto_bucket = random.choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        src_object = self.random_key()
        dst_object = src_object + '-dest'

        content = random_bytes(300 * 1024)

        # 上传源文件
        crypto_bucket.put_object(src_object, content)

        # upload part copy到目标文件
        # upload_id = self.bucket.init_multipart_upload(dst_object).upload_id
        # self.assertRaises(oss2.exceptions.NotImplemented, self.bucket.upload_part_copy, self.bucket.bucket_name,
        # src_object, (0, 100 * 1024 - 1), dst_object, upload_id, 1)
        # abort_result = self.bucket.abort_multipart_upload(dst_object, upload_id)
        # self.assertTrue(abort_result.status == 204)

        data_size = len(content)
        part_size = 100 * 1024
        context = models.MultipartUploadCryptoContext(data_size, part_size)
        upload_id = crypto_bucket.init_multipart_upload(dst_object, upload_context=context).upload_id
        self.assertRaises(oss2.exceptions.ClientError, crypto_bucket.upload_part_copy,
                          crypto_bucket.bucket_name,
                          src_object, (0, 100 * 1024 - 1), dst_object, upload_id, 1)

        abort_result = crypto_bucket.abort_multipart_upload(dst_object, upload_id)
        self.assertTrue(abort_result.status == 204)

    def test_crypto_multipart_concurrency(self):
        crypto_bucket = random.choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key1 = self.random_key()
        key1_content_1 = random_bytes(100 * 1024)
        key1_content_2 = random_bytes(100 * 1024)
        key1_content_3 = random_bytes(100 * 1024)
        key1_content = [key1_content_1, key1_content_2, key1_content_3]

        key1_parts = []
        key1_data_size = 1024 * 300
        key1_part_size = 1024 * 100

        context1 = models.MultipartUploadCryptoContext(key1_data_size, key1_part_size)
        key1_init_result = crypto_bucket.init_multipart_upload(key1, upload_context=context1)
        self.assertTrue(key1_init_result.status == 200)
        key1_upload_id = key1_init_result.upload_id

        key2 = self.random_key()
        key2_content_1 = random_bytes(200 * 1024)
        key2_content_2 = random_bytes(200 * 1024)
        key2_content_3 = random_bytes(100 * 1024)
        key2_content = [key2_content_1, key2_content_2, key2_content_3]

        key2_parts = []
        key2_data_size = 1024 * 500
        key2_part_size = 1024 * 200

        context2 = models.MultipartUploadCryptoContext(key2_data_size, key2_part_size)
        key2_init_result = crypto_bucket.init_multipart_upload(key2, upload_context=context2)
        self.assertTrue(key2_init_result.status == 200)
        key2_upload_id = key2_init_result.upload_id

        for i in range(3):
            key1_upload_result = crypto_bucket.upload_part(key1, key1_upload_id, i + 1, key1_content[i],
                                                           upload_context=context1)
            key1_parts.append(oss2.models.PartInfo(i + 1, key1_upload_result.etag,
                                                   part_crc=key1_upload_result.crc))
            self.assertTrue(key1_upload_result.status == 200)
            self.assertTrue(key1_upload_result.crc is not None)

            key2_upload_result = crypto_bucket.upload_part(key2, key2_upload_id, i + 1, key2_content[i],
                                                           upload_context=context2)
            key2_parts.append(oss2.models.PartInfo(i + 1, key2_upload_result.etag, size=len(key2_content[i]),
                                                   part_crc=key2_upload_result.crc))
            self.assertTrue(key2_upload_result.status == 200)
            self.assertTrue(key2_upload_result.crc is not None)

        key1_complete_result = crypto_bucket.complete_multipart_upload(key1, key1_upload_id, key1_parts)
        self.assertTrue(key1_complete_result.status == 200)

        key1_get_result = crypto_bucket.get_object(key1)
        self.assertTrue(key1_get_result.status == 200)
        key1_content_got = key1_get_result.read()
        self.assertEqual(key1_content_1, key1_content_got[0:102400])
        self.assertEqual(key1_content_2, key1_content_got[102400:204800])
        self.assertEqual(key1_content_3, key1_content_got[204800:307200])

        key2_complete_result = crypto_bucket.complete_multipart_upload(key2, key2_upload_id, key2_parts)
        self.assertTrue(key2_complete_result.status == 200)

        key2_get_result = crypto_bucket.get_object(key2)
        self.assertTrue(key2_get_result.status == 200)
        key2_content_got = key2_get_result.read()
        self.assertEqual(key2_content_1, key2_content_got[0:204800])
        self.assertEqual(key2_content_2, key2_content_got[204800:409600])
        self.assertEqual(key2_content_3, key2_content_got[409600:512000])

    '''
    def test_crypto_upload_invalid_part_content(self):
        crypto_bucket = random.choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key()
        content_invalid = random_bytes(random.randint(1, 100 * 1024 - 1))

        data_size = 250 * 1024 - 1
        part_size = 1024 * 100

        init_result = crypto_bucket.init_multipart_upload(key, data_size, part_size)
        self.assertTrue(init_result.status == 200)
        upload_id = init_result.upload_id

        # invalid part size
        self.assertRaises(oss2.exceptions.InvalidEncryptionRequest, crypto_bucket.upload_part, key, upload_id, 1,
                          content_invalid)

        abort_result = crypto_bucket.abort_multipart_upload(key, upload_id)
        self.assertTrue(abort_result.status == 204)
    '''

    '''
    def test_crypto_upload_invalid_last_part_content(self):
        crypto_bucket = random.choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key()
        content_1 = random_bytes(100 * 1024)
        content_2 = random_bytes(100 * 1024)
        content_3 = random_bytes(50 * 1024)
        content = [content_1, content_2, content_3]
        content_invalid = content_3[0:random.randint(1, 50 * 1024 - 1)]

        parts = []
        data_size = 1024 * 250
        part_size = 1024 * 100

        init_result = crypto_bucket.init_multipart_upload(key, data_size, part_size)
        self.assertTrue(init_result.status == 200)
        upload_id = init_result.upload_id

        for i in range(2):
            upload_result = crypto_bucket.upload_part(key, upload_id, i + 1, content[i])
            parts.append(
                oss2.models.PartInfo(i + 1, upload_result.etag, part_crc=upload_result.crc))
            self.assertTrue(upload_result.status == 200)
            self.assertTrue(upload_result.crc is not None)

        self.assertRaises(oss2.exceptions.InvalidEncryptionRequest, crypto_bucket.upload_part, key, upload_id, 3,
                          content_invalid)

        abort_result = crypto_bucket.abort_multipart_upload(key, upload_id)
        self.assertTrue(abort_result.status == 204)
    '''

    '''
    def test_crypto_upload_invalid_part_number(self):
        crypto_bucket = random.choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key()
        content_1 = random_bytes(100 * 1024)

        data_size = 1024 * 250
        part_size = 1024 * 100
        invalid_part_num = random.randint(4, 100)

        init_result = crypto_bucket.init_multipart_upload(key, data_size, part_size)
        self.assertTrue(init_result.status == 200)
        upload_id = init_result.upload_id

        self.assertRaises(oss2.exceptions.InvalidEncryptionRequest, crypto_bucket.upload_part, key, upload_id,
                          invalid_part_num, content_1)

        abort_result = crypto_bucket.abort_multipart_upload(key, upload_id)
        self.assertTrue(abort_result.status == 204)
    '''

    '''
    def test_crypto_complete_multipart_miss_parts(self):
        crypto_bucket = random.choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key()
        content_1 = random_bytes(100 * 1024)
        content_2 = random_bytes(100 * 1024)
        content_3 = random_bytes(50 * 1024)
        content = [content_1, content_2, content_3]

        parts = []
        data_size = 1024 * 250
        part_size = 1024 * 100

        init_result = crypto_bucket.init_multipart_upload(key, data_size, part_size)
        self.assertTrue(init_result.status == 200)
        upload_id = init_result.upload_id

        for i in range(2):
            upload_result = crypto_bucket.upload_part(key, upload_id, i + 1, content[i])
            parts.append(
                oss2.models.PartInfo(i + 1, upload_result.etag, size=len(content[i]), part_crc=upload_result.crc))
            self.assertTrue(upload_result.status == 200)
            self.assertTrue(upload_result.crc is not None)

        self.assertRaises(oss2.exceptions.InvalidEncryptionRequest, crypto_bucket.complete_multipart_upload,
                          key,
                          upload_id, parts)

        abort_result = crypto_bucket.abort_multipart_upload(key, upload_id)
        self.assertTrue(abort_result.status == 204)
    '''

    '''
    def test_crypto_list_parts(self):
        crypto_bucket = random.choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key()
        content = random_bytes(100 * 1024)

        data_size = 1024 * 300
        part_size = 1024 * 100

        init_result = crypto_bucket.init_multipart_upload(key, data_size, part_size)
        self.assertTrue(init_result.status == 200)
        upload_id = init_result.upload_id

        upload_result = crypto_bucket.upload_part(key, upload_id, 1, content)
        self.assertTrue(upload_result.status == 200)
        self.assertTrue(upload_result.crc is not None)

        list_result = crypto_bucket.list_parts(key, upload_id)
        self.assertTrue(list_result.status == 200)

        self.assertEqual(data_size, list_result.client_encryption_data_size)
        self.assertEqual(part_size, list_result.client_encryption_part_size)

        abort_result = crypto_bucket.abort_multipart_upload(key, upload_id)
        self.assertTrue(abort_result.status == 204)
    '''



# -*- coding: utf-8 -*-

import unittest
import xml.etree.ElementTree as ElementTree
import oss2
from oss2.models import ReplicationRule
from oss2.xml_utils import _find_tag, _find_bool
from oss2.xml_utils import (parse_get_bucket_info,
                            parse_get_bucket_replication_result,
                            parse_get_bucket_replication_location_result,
                            parse_get_bucket_replication_progress_result)
from .common import MockResponse



class TestXmlUtils(unittest.TestCase):
    def test_find_tag(self):
        body = '''
        <Test>
            <Grant>private</Grant>
        </Test>'''

        root = ElementTree.fromstring(body)

        grant = _find_tag(root, 'Grant')
        self.assertEqual(grant, 'private')

        self.assertRaises(RuntimeError, _find_tag, root, 'none_exist_tag')

    def test_find_bool(self):
        body = '''
        <Test>
            <BoolTag1>true</BoolTag1>
            <BoolTag2>false</BoolTag2>
        </Test>'''

        root = ElementTree.fromstring(body)

        tag1 = _find_bool(root, 'BoolTag1')
        tag2 = _find_bool(root, 'BoolTag2')
        self.assertEqual(tag1, True)
        self.assertEqual(tag2, False)

        self.assertRaises(RuntimeError, _find_bool, root, 'none_exist_tag')

    def test_parse_get_bucket_info(self):
        body = '''
        <BucketInfo>
            <Bucket>
                <CreationDate>2013-07-31T10:56:21.000Z</CreationDate>
                <ExtranetEndpoint>oss-cn-hangzhou.aliyuncs.com</ExtranetEndpoint>
                <IntranetEndpoint>oss-cn-hangzhou-internal.aliyuncs.com</IntranetEndpoint>
                <Location>oss-cn-hangzhou</Location>
                <Name>oss-example</Name>
                <StorageClass>IA</StorageClass>
                <Owner>
                    <DisplayName>username</DisplayName>
                    <ID>27183473914****</ID>
                </Owner>
                <AccessControlList>
                    <Grant>private</Grant>
                </AccessControlList>
            </Bucket>
        </BucketInfo>
        '''
        headers = oss2.CaseInsensitiveDict({
            'Server': 'AliyunOSS',
            'Date': 'Fri, 11 Dec 2015 11:40:30 GMT',
            'Content-Length': len(body),
            'Connection': 'keep-alive',
            'x-oss-request-id': '566AB62EB06147681C283D73',
            'ETag': '7AE1A589ED6B161CAD94ACDB98206DA6'
        })
        resp =  MockResponse(200, headers, body)

        result = oss2.models.GetBucketInfoResult(resp)
        parse_get_bucket_info(result, body)
        self.assertEqual(result.location, 'oss-cn-hangzhou')
        self.assertIsNone(result.data_redundancy_type)
        self.assertIsNone(result.comment)
        self.assertIsNone(result.versioning_status)
        self.assertIsNone(result.bucket_encryption_rule)

    def test_parse_get_bucket_replication(self):
        body = '''<ReplicationConfiguration>
  <Rule>
    <ID>test_replication_1</ID>
    <PrefixSet>
      <Prefix>source_image</Prefix>
      <Prefix>video</Prefix>
    </PrefixSet>
    <Action>PUT</Action>
    <Destination>
      <Bucket>target-bucket</Bucket>
      <Location>oss-ap-south-1</Location>
      <TransferType>oss_acc</TransferType>
    </Destination>
    <Status>doing</Status>
    <HistoricalObjectReplication>enabled</HistoricalObjectReplication>
    <SyncRole>aliyunramrole</SyncRole>
    <SourceSelectionCriteria>
      <SseKmsEncryptedObjects>
        <Status>Enabled</Status>
      </SseKmsEncryptedObjects>
    </SourceSelectionCriteria>
    <EncryptionConfiguration>
      <ReplicaKmsKeyID>c2ee80d6-1111-1111-1111-a3644797b566</ReplicaKmsKeyID>
    </EncryptionConfiguration>
  </Rule>
</ReplicationConfiguration>
        '''
        headers = oss2.CaseInsensitiveDict({
            'Server': 'AliyunOSS',
            'Date': 'Fri, 11 Dec 2015 11:40:30 GMT',
            'Content-Length': len(body),
            'Connection': 'keep-alive',
            'x-oss-request-id': '566AB62EB06147681C283D73',
            'ETag': '7AE1A589ED6B161CAD94ACDB98206DA6'
        })
        resp = MockResponse(200, headers, body)

        result = oss2.models.GetBucketReplicationResult(resp)
        parse_get_bucket_replication_result(result, body)
        self.assertEqual(1, len(result.rule_list))
        rule = result.rule_list[0]

        self.assertEqual('test_replication_1', rule.rule_id)
        self.assertEqual('target-bucket', rule.target_bucket_name)
        self.assertEqual('oss-ap-south-1', rule.target_bucket_location)
        self.assertEqual('oss_acc', rule.target_transfer_type)
        self.assertEqual(2, len(rule.prefix_list))
        self.assertTrue('source_image' in rule.prefix_list)
        self.assertTrue('video' in rule.prefix_list)
        self.assertEqual(1, len(rule.action_list))
        self.assertEqual('PUT', rule.action_list[0])
        self.assertTrue(rule.is_enable_historical_object_replication)
        self.assertEqual(ReplicationRule.DOING, rule.status)
        self.assertEqual('aliyunramrole', rule.sync_role_name)
        self.assertEqual('c2ee80d6-1111-1111-1111-a3644797b566', rule.replica_kms_keyid)
        self.assertEqual('Enabled', rule.sse_kms_encrypted_objects_status)

    def test_parse_get_bucket_replication_location(self):
        body = '''<ReplicationLocation>
  <Location>oss-cn-beijing</Location>
  <Location>oss-cn-qingdao</Location>
  <Location>oss-cn-shenzhen</Location>
  <Location>oss-cn-hongkong</Location>
  <Location>oss-us-west-1</Location>
  <LocationTransferTypeConstraint>
    <LocationTransferType>
      <Location>oss-cn-hongkong</Location>
        <TransferTypes>
          <Type>oss_acc</Type>          
        </TransferTypes>
      </LocationTransferType>
      <LocationTransferType>
        <Location>oss-us-west-1</Location>
        <TransferTypes>
          <Type>oss_acc</Type>
        </TransferTypes>
      </LocationTransferType>
    </LocationTransferTypeConstraint>
  </ReplicationLocation>'''
        headers = oss2.CaseInsensitiveDict({
            'Server': 'AliyunOSS',
            'Date': 'Fri, 11 Dec 2015 11:40:30 GMT',
            'Content-Length': len(body),
            'Connection': 'keep-alive',
            'x-oss-request-id': '566AB62EB06147681C283D73',
            'ETag': '7AE1A589ED6B161CAD94ACDB98206DA6'
        })
        resp = MockResponse(200, headers, body)

        result = oss2.models.GetBucketReplicationLocationResult(resp)
        parse_get_bucket_replication_location_result(result, body)
        self.assertEqual(5, len(result.location_list))
        self.assertTrue('oss-cn-beijing' in result.location_list)
        self.assertTrue('oss-cn-qingdao' in result.location_list)
        self.assertTrue('oss-cn-shenzhen' in result.location_list)
        self.assertTrue('oss-cn-hongkong' in result.location_list)
        self.assertTrue('oss-us-west-1' in result.location_list)
        self.assertEqual(2, len(result.location_transfer_type_list))
        self.assertTrue('oss-cn-hongkong' in [result.location_transfer_type_list[0].location,
                                              result.location_transfer_type_list[1].location])
        self.assertTrue('oss-us-west-1' in [result.location_transfer_type_list[0].location,
                                            result.location_transfer_type_list[1].location])
        self.assertEqual('oss_acc', result.location_transfer_type_list[0].transfer_type)
        self.assertEqual('oss_acc', result.location_transfer_type_list[1].transfer_type)

    def test_parse_get_bucket_replication_progress(self):
        body = '''<ReplicationProgress>
 <Rule>
   <ID>test_replication_1</ID>
   <PrefixSet>
    <Prefix>source_image</Prefix>
    <Prefix>video</Prefix>
   </PrefixSet>
   <Action>PUT,ABORT,DELETE</Action>
   <Destination>
    <Bucket>target-bucket</Bucket>
    <Location>oss-cn-beijing</Location>
    <TransferType>oss_acc</TransferType>
   </Destination>
   <Status>doing</Status>
   <HistoricalObjectReplication>enabled</HistoricalObjectReplication>
   <Progress>
    <HistoricalObject>0.85</HistoricalObject>
    <NewObject>2015-09-24T15:28:14.000Z</NewObject>
   </Progress>
 </Rule>
</ReplicationProgress>'''
        headers = oss2.CaseInsensitiveDict({
            'Server': 'AliyunOSS',
            'Date': 'Fri, 11 Dec 2015 11:40:30 GMT',
            'Content-Length': len(body),
            'Connection': 'keep-alive',
            'x-oss-request-id': '566AB62EB06147681C283D73',
            'ETag': '7AE1A589ED6B161CAD94ACDB98206DA6'
        })
        resp = MockResponse(200, headers, body)

        result = oss2.models.GetBucketReplicationProgressResult(resp)
        parse_get_bucket_replication_progress_result(result, body)
        progress = result.progress
        self.assertEqual('test_replication_1', progress.rule_id)
        self.assertEqual('target-bucket', progress.target_bucket_name)
        self.assertEqual('oss-cn-beijing', progress.target_bucket_location)
        self.assertEqual('oss_acc', progress.target_transfer_type)
        self.assertEqual(2, len(progress.prefix_list))
        self.assertTrue('source_image' in progress.prefix_list)
        self.assertTrue('video' in progress.prefix_list)

        self.assertEqual(3, len(progress.action_list))
        self.assertTrue('PUT' in progress.action_list)
        self.assertTrue('DELETE' in progress.action_list)
        self.assertTrue('ABORT' in progress.action_list)
        self.assertTrue(progress.is_enable_historical_object_replication)
        self.assertEqual('doing', progress.status)
        self.assertEqual(0.85, progress.historical_object_progress)
        self.assertEqual('2015-09-24T15:28:14.000Z', progress.new_object_progress)


if __name__ == '__main__':
    unittest.main()




# -*- coding: utf-8 -*-

import os

import oss2


# 以下代码展示了文件下载的用法，如下载文件、范围下载、断点续传下载等。


# 首先初始化AccessKeyId、AccessKeySecret、Endpoint等信息。
# 通过环境变量获取，或者把诸如“<你的AccessKeyId>”替换成真实的AccessKeyId等。
#
# 以杭州区域为例，Endpoint可以是：
#   http://oss-cn-hangzhou.aliyuncs.com
#   https://oss-cn-hangzhou.aliyuncs.com
# 分别以HTTP、HTTPS协议访问。
access_key_id = os.getenv('OSS_TEST_ACCESS_KEY_ID', '<你的AccessKeyId>')
access_key_secret = os.getenv('OSS_TEST_ACCESS_KEY_SECRET', '<你的AccessKeySecret>')
bucket_name = os.getenv('OSS_TEST_BUCKET', '<你的Bucket>')
endpoint = os.getenv('OSS_TEST_ENDPOINT', '<你的访问域名>')


# 确认上面的参数都填写正确了
for param in (access_key_id, access_key_secret, bucket_name, endpoint):
    assert '<' not in param, '请设置参数：' + param


# 创建Bucket对象，所有Object相关的接口都可以通过Bucket对象来进行
bucket = oss2.Bucket(oss2.Auth(access_key_id, access_key_secret), endpoint, bucket_name)

key = 'motto.txt'
content = oss2.to_bytes('a' * 1024 * 1024)
filename = 'download.txt'

# 上传文件
bucket.put_object(key, content, headers={'content-length': str(1024 * 1024)})

"""
文件下载
"""

# 下载文件
result = bucket.get_object(key)

# 验证一下
content_got = b''
for chunk in result:
    content_got += chunk
assert content_got == content

# 下载到本地文件
result = bucket.get_object_to_file(key, filename)

# 验证一下
with open(filename, 'rb') as fileobj:
    assert fileobj.read() == content

"""
范围下载
"""

# 范围下载，如果指定的范围无效，则下载整个文件
result = bucket.get_object(key, byte_range=(0, 1023))

# 验证一下
content_got = b''
for chunk in result:
    content_got += chunk
assert content_got == oss2.to_bytes('a'*1024)


# 范围下载到本地文件
result = bucket.get_object_to_file(key, filename, byte_range=(1024, 2047))

# 验证一下
with open(filename, 'rb') as fileobj:
    assert fileobj.read() == oss2.to_bytes('a'*1024)


"""
断点续传下载
"""

# 断点续传下载
oss2.resumable_download(bucket, key, filename,
                        multiget_threshold=200*1024,
                        part_size=100*1024,
                        num_threads=3)

# 验证一下
with open(filename, 'rb') as fileobj:
    assert fileobj.read() == content

# 清理文件
os.remove(filename)



# -*- coding: utf-8 -*-

import os
import oss2
from oss2.headers import OSS_REQUEST_PAYER

# 以下代码展示了第三方付费请求object的示例
# 在此之前需要Bucket的拥有者给第三方请授权，并开启请求者付费模式。

# 首先初始化AccessKeyId、AccessKeySecret、Endpoint等信息。
# 通过环境变量获取，或者把诸如“<你的AccessKeyId>”替换成真实的AccessKeyId等。
access_key_id = os.getenv('OSS_TEST_ACCESS_KEY_ID', '<你的AccessKeyId>')
access_key_secret = os.getenv('OSS_TEST_ACCESS_KEY_SECRET', '<你的AccessKeySecret>')
bucket_name = os.getenv('OSS_TEST_BUCKET', '<你要请求的Bucket名称>')
endpoint = os.getenv('OSS_TEST_ENDPOINT', '<你的访问域名>')

# 确认上面的参数都填写正确了
for param in (access_key_id, access_key_secret, bucket_name, endpoint):
    assert '<' not in param, '请设置参数：' + param

# 创建Bucket对象，所有Object相关的接口都可以通过Bucket对象来进行
bucket = oss2.Bucket(oss2.Auth(access_key_id, access_key_secret), endpoint, bucket_name)

object_name = 'test-request-object'

# header中指定请求者付费
headers = dict()
headers[OSS_REQUEST_PAYER] = "requester"

# 上传文件, 需要指定header
result = bucket.put_object(object_name, 'test-content', headers=headers)
print('http response status: ', result.status)

# 删除文件, 需要指定header.
result = bucket.delete_object(object_name, headers=headers);
print('http response status: ', result.status)


# -*- coding: utf-8 -*-

import json
import os

from PIL import Image

import oss2


# 以下代码展示了图片服务的基本用法。更详细应用请参看官网文档 https://help.aliyun.com/document_detail/32206.html

# 首先初始化AccessKeyId、AccessKeySecret、Endpoint等信息。
# 通过环境变量获取，或者把诸如“<你的AccessKeyId>”替换成真实的AccessKeyId等。
#
# 以杭州区域为例，Endpoint可以是：
#   http://oss-cn-hangzhou.aliyuncs.com
#   https://oss-cn-hangzhou.aliyuncs.com
# 分别以HTTP、HTTPS协议访问。
access_key_id = os.getenv('OSS_TEST_ACCESS_KEY_ID', '<你的AccessKeyId>')
access_key_secret = os.getenv('OSS_TEST_ACCESS_KEY_SECRET', '<你的AccessKeySecret>')
bucket_name = os.getenv('OSS_TEST_BUCKET', '<你的Bucket>')
endpoint = os.getenv('OSS_TEST_ENDPOINT', '<你的访问域名>')


# 确认上面的参数都填写正确了
for param in (access_key_id, access_key_secret, bucket_name, endpoint):
    assert '<' not in param, '请设置参数：' + param

def get_image_info(image_file):
    """获取本地图片信息
    :param str image_file: 本地图片
    :return tuple: a 3-tuple(height, width, format).
    """
    im = Image.open(image_file)
    return im.height, im.width, im.format

# 创建Bucket对象，所有Object相关的接口都可以通过Bucket对象来进行
bucket = oss2.Bucket(oss2.Auth(access_key_id, access_key_secret), endpoint, bucket_name)

key = 'example.jpg'
new_pic = 'new-example.jpg'

# 上传示例图片
bucket.put_object_from_file(key, 'example.jpg')

# 获取图片信息
result = bucket.get_object(key, process='image/info')

json_content = result.read()
decoded_json = json.loads(oss2.to_unicode(json_content))
assert int(decoded_json['ImageHeight']['value']) == 267
assert int(decoded_json['ImageWidth']['value']) == 400
assert int(decoded_json['FileSize']['value']) == 21839
assert decoded_json['Format']['value'] == 'jpg'

# 图片缩放
process = "image/resize,m_fixed,w_100,h_100"
bucket.get_object_to_file(key, new_pic, process=process)
info = get_image_info(new_pic)
assert info[0] == 100
assert info[1] == 100
assert info[2] == 'JPEG'

# 图片裁剪
process = "image/crop,w_100,h_100,x_100,y_100,r_1"
bucket.get_object_to_file(key, new_pic, process=process)
info = get_image_info(new_pic)
assert info[0] == 100
assert info[1] == 100
assert info[2] == 'JPEG'

# 图片旋转
process = "image/rotate,90"
bucket.get_object_to_file(key, new_pic, process=process)
info = get_image_info(new_pic)
assert info[0] == 400
assert info[1] == 267
assert info[2] == 'JPEG'

# 图片锐化
process = "image/sharpen,100"
bucket.get_object_to_file(key, new_pic, process=process)
info = get_image_info(new_pic)
assert info[0] == 267
assert info[1] == 400
assert info[2] == 'JPEG'

# 图片加文字水印
process = "image/watermark,text_SGVsbG8g5Zu-54mH5pyN5YqhIQ"
bucket.get_object_to_file(key, new_pic, process=process)
info = get_image_info(new_pic)
assert info[0] == 267
assert info[1] == 400
assert info[2] == 'JPEG'

# 图片格式转换
process = "image/format,png"
bucket.get_object_to_file(key, new_pic, process=process)
info = get_image_info(new_pic)
assert info[0] == 267
assert info[1] == 400
assert info[2] == 'PNG'

# 删除示例图片
bucket.delete_object(key)
# 清除本地文件
os.remove(new_pic)
